package gwpkg;

import java.io.Serializable;
import javax.persistence.Entity;
import javax.ejb.Stateful;
import javax.persistence.*;
import org.jdom.Document;  
import org.jdom.Element;  
import org.jdom.output.Format; 
import org.jdom.output.XMLOutputter;

/**
 * Classe que representa el model de dades de l'entitat CONFIGURACIO <P>
 * Una instncia d'aquesta entitat es correspon a un fitxer de configuraci de WPKG
 * Defineix els parmetres amb que s'execur l'script d'installaci de programari
 * a l'equip local (ws.js).  
 */
@Entity
@Stateful
@Table(name="configuracions")
public class Configuracio implements Serializable{
	private static final long serialVersionUID = 1L;

/** Codi identificadoer de la configuraci. s autonumric. */	
@Id
@GeneratedValue(strategy=GenerationType.IDENTITY)
private Integer idConfiguracio; 

/** Nom de la configuraci. Descriptor mnemotcnic d'entre 0 i 15 caracters.*/ 
private String nomConfiguracio;

/** Descripci de la configuraci. Descripci general de les especifitats d'una configuraci.
 * Mxim 100 caracters */ 
private String descripcioConfiguracio;

/** Estat de la configuraci. Habilitada/Deshabilitada.
 * Si est habilitada es mostrar a els grids de selecci de configuracions.
 * Si est deshabilitada no es mostrar als grids de selecci.
 * L'estat deshabilitat no comporta cap altre comportament.
 * BOOLEAN */ 
private Boolean inhabilitada=false;

/** Parmetre WPKG forceInstall.
 * True : Force installation over existing packages.
 * False: Skip package installation if package is already installed.
 * Default value      : false
 * Command-line switch: /forceinstall
 */
private Boolean forceInstall=false;

/** Parmetre WPKG quitonerror.
 * True : Force the script to immediately quit on error.
 * False: Errors are logged but WPKG continues processing.
 * Default value      : false
 * Command-line switch: /quitonerror
 */
private Boolean quitonerror=false;

/** Parmetre WPKG debug.
 *True : Enable debug mode. Prints lots of ugly debug messages to event log.
 * False: Disable debug logging to event log.
 * NOTE: This flag is independent from the log level setting. So you can even
 *       enable debug logging to log files without setting this flag to true
 *       which prevents your event log to be flooded.
 * Default value      : false
 * Command-line switch: /debug
 */
private Boolean debug=false;

/** Parmetre WPKG dryrun.
 * True : Enable dryrun mode. Does not apply any changes to the system.
 *        Enables debug output and disables reboot.
 * False: Apply changes to the system.
 * Default value      : false
 * Command-line switch: /dryrun
 */
private Boolean dryrun=false;

/** Parmetre WPKG quiet.
* Should be set to true in case of unattended run (WPKG service).
* True : Disable all log messages printed to the console (cscript) or
*        displayed as dialog boxes (wscript).
* False: Alerts the user about ongoing activity using windows messaging.
* Default value      : false
* Command-line switch: /quiet
*/
private Boolean quiet=true;

/** Parmetre WPKG quiet.
* True : Disable user notification about WPKG actions using windows messaging.
*        Should be set to true in case of unattended run (WPKG service)
* False: Alerts the user about ongoing activity using windows messaging.
* Default value      : false
* Command-line switch: /nonotify
*/
private Boolean nonotify=true;

/** Parmetre WPKG notificationDisplayTime.
* Defines how long a user notification is displayed to the user. After
* timeout has been reached the message will be closed. Specify 0 in order to
* specify that messages are never closed automatically.
* Note: This only works on Windows Vista or newer where msg.exe is available.
* Default value      : 10
* Command-line switch: -
 */
private Integer notificationDisplayTime=1024;

/** Parmetre WPKG noreboot.
* True : System does not reboot regardless of need.
* False: Reboot the system as specified by packages.
* Default value      : false
* Command-line switch: /noreboot
 */
private Boolean noreboot=false;

/** Parmetre WPKG noRunningState.
* True : Disable export of running state to Windows registry at
*        HKCU\Software\WPKG\running
* False: Export running state to Windows registry at HKCU\Software\WPKG\running
* Default value      : false
* Command-line switch: /norunningstate
 */
private Boolean noRunningState=false;

/** Parmetre WPKG caseSensitivity.
* True : Matching of package and profile IDs is case sensitive.
* False: Disable case sensitivity for package and profile IDs.
*        WPKG will consider the package ID "SomePackage" and "SoMePaCkAgE"
*        the same.
* Default value      : true
* Command-line switch: /ignoreCase
 */
private Boolean caseSensitivity=true;

/** Parmetre WPKG rebootCmd.
* Use the specified command for rebooting, either with full path or relative
* to the location of wpkg.js.
* Setting rebootCmd to "special" will use tools\psshutdown.exe from
* from www.sysinternals.com - if it exists
* Any other value will make WPKG to try executing the command specified as
* <command> and then exit with code 3010.
* Default value      : "standard"
* Command-line switch: /rebootcmd:<command>
 */
private String rebootCmd="standard2";

/** Parmetre WPKG noForcedRemove.
* True : Disable forced removal of packages if they are removed from the
*        profile AND the package database.
*  NOTE: A client might continuously try to uninstall a package if an
*        uninstall command fails.
* False: 
* Default value      : false
* Command-line switch: /noforcedremove
 */
private Boolean noForcedRemove=false;

/** Parmetre WPKG noForcedRemove.
* True : Allows to disable removing of packages. If used in conjunction with
*        the /synchronize parameter it will just add packages but never remove
*        them. Instead of removing a list of packages which would have been
*        removed during that session is printed on exit. Packages are not
*        removed from the local settings database (wpkg.xml). Therefore it
*        will contain a list of all packages ever installed.
*  NOTE: Each package which is requested to be removed (manually or by
*        a synchronization request) will be checked for its state by executing
*        its included package checks. If the package has been removed manually
*        it will also be removed from the settings database. Due to the fact
*        that packages whithout checks always return 'false' for during the
*        install-status check these packages will disappear from the local
*        wpkg.xml.
* False: Remove packages from the system if they are not listed within the
*        profile any more.
* Default value      : false
* Command-line switch: /noremove
 */
private Boolean noRemove=false;

/** Parmetre WPKG sendStatus.
* Controls weather WPKG prints some information about its progress to
* STDOUT. This output can be read by another program (GUI) to display some
* progress bar or additional status information to the user.
* True : Enable status output on STDOUT.
* False: Disable status output on STDOUT.
* Default value      : false
* Command-line switch: /sendStatus
 */
private Boolean sendStatus=false;

/** Parmetre WPKG noUpgradeBeforeRemove.
* Usually WPKG upgrades a package to the latest available version before it
* removes the package. This allows administrators to fix bugs in the package
* and assure proper removal.
* However this feature can lead to (sometimes unexpected) re-execution of
* packages. For example if execution=always is used the package will run
* once again before it is finally removed.
* It's advised to leave this option on default unless you really know what
* you're doing.
* True : Disables the upgrade-before-remove feature
* False: Leave the upgrade-before-remove feature enabled
* Default value      : false
* Command-line switch: /noUpgradeBeforeRemove
 */
private Boolean noUpgradeBeforeRemove=false;

/** Parmetre WPKG logAppend.
* True : Log files are appended instead of overwritten. Please note that
*        log files could grow quite large if debug level logging is enabled.
*        Depending on the pattern (date and time used) a new log file might
*        be written on each run.
* False: Log files are overwritten on each run. This is the default setting
*        and it is aimed for the target to keep only the log of the most
*        recent WPKG run.
*/
private Boolean logAppend=false;

/** Parmetre WPKG logLevel / logErrors.
*  s un clcul obtingut a partir dels valors del segents atributs de GWPKG
*  	logErrors, logWarnings, logInformation, logSuccess i logFailure 
*  Aquests atributs no existeixen a WPKG. Es fan servir a GWPKG per calcular 
*  	logLevel
* Log level is defined as a bitmask. Just sum up the values of each log
* severity you would like to include within the log file and add this value
* to your config.xml or specify it at /logLevel:<num>.
* Specify 0 to disable logging.
*  1: log errors only
*  2: log warnings
*  4: log information
*  8: log audit success
* 16: log audit failure
* Default value      : "0xFF"
* Command-line switch: /logLevel:<level>
 */
private Boolean logErrors=true;

/** Parmetre WPKG logLevel / logWarnings.
 * Veure atribut logErrors
 */
private Boolean logWarnings=true;

/** Parmetre WPKG logLevel / logInformation.
 * Veure atribut logErrors
 */
private Boolean logInformation=true;

/** Parmetre WPKG logLevel / logSuccess.
 * Veure atribut logErrors
 */
private Boolean logSuccess=true;

/** Parmetre WPKG logLevel / logFailure.
 * Veure atribut logErrors
 */
private Boolean logFailure=true;

/** Parmetre WPKG logFilePath
* Path where the logfiles are written to. This might be an UNC path on the
* network as well as a local path. Environment variables are expanded.
* Examples:
*     <param name='log_file_path' value='%TEMP%' />
*     <param name='log_file_path' value='\\\\server\\share\\dir' />
*
* Default value      : "%TEMP%"
* Command-line switch: /log_file_path:<path>
 */
private String logFilePath="%TEMP%";

/** Parmetre WPKG logfilePattern
* Pattern to generate the log file name.
* Recognized patterns:
* [HOSTNAME] replaced by the executing hostname
* [PROFILE]  replaced by the applying profile name
* [YYYY]     replaced by year (4 digits)
* [MM]       replaced by month number (2 digits)
* [DD]       replaced by the day of the month (2 digits)
* [hh]       replaced by hour of the day (24h format, 2 digits)
* [mm]       replaced by minutes (2 digits)
* [ss]       replaced by seconds (2 digits)
*
* Examples:
*     "wpkg-[YYYY]-[MM]-[DD]-[HOSTNAME].log" results in a name like
*         "wpkg-2007-11-04-myhost.log"
*
* NOTE: Using [PROFILE] causes all log messages from before reading
*       profiles.xml to be temporarily written to the local %TEMP% folder.
*       So they might appear on the final log file with some delay.
*
* NOTE: The default pattern allows tracing of last execution without polluting
*       the log directory. Subsequent executions will overwrite the same log
*       file.
*
* Default value      : "wpkg-[HOSTNAME].log"
* Command-line switch: /logfilePattern:<pattern>
 */
private String logFilePattern="wpkg-[HOSTNAME].log";

/**Constructor buit exigit per l'especificaci JPA2/EJB3
 */
public Configuracio(){
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Integer getIdConfiguracio(){
	return idConfiguracio;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getNomConfiguracio(){
	return nomConfiguracio;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNomConfiguracio(String nomConfiguracio){
	this.nomConfiguracio=nomConfiguracio;
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getDescripcioConfiguracio(){
	return descripcioConfiguracio;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setInhabilitada(Boolean inhabilitada){
	this.inhabilitada=inhabilitada;
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getInhabilitada(){
	return inhabilitada;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setDescripcioConfiguracio(String descripcioConfiguracio){
	this.descripcioConfiguracio=descripcioConfiguracio;
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getForceInstall(){
	return forceInstall;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setForceInstall(Boolean forceInstall){
	this.forceInstall=forceInstall;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setQuitonerror(Boolean quitonerror){
	this.quitonerror=quitonerror;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getQuitonerror(){
	return this.quitonerror;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setDebug(Boolean debug){
	this.debug=debug;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getDebug(){
	return this.debug;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setDryrun(Boolean dryrun){
	this.dryrun=dryrun;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getdryrun(){
	return this.dryrun;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setQuiet(Boolean quiet){
	this.quiet=quiet;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getQuiet(){
	return this.quiet;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNonotify(Boolean nonotify){
	this.nonotify=nonotify;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNonotify(){
	return this.nonotify;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNotificationDisplayTime(Integer notificationDisplayTime){
	this.notificationDisplayTime=notificationDisplayTime;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Integer getNotificationDisplayTime(){
	return this.notificationDisplayTime;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNoreboot(Boolean noreboot){
	this.noreboot=noreboot;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNoreboot(){
	return this.noreboot;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNoRunningState(Boolean noRunningState){
	this.noRunningState=noRunningState;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNoRunningState(){
	return this.noRunningState;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setCaseSensitivity(Boolean caseSensitivity){
	this.caseSensitivity=caseSensitivity;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getCaseSensitivity(){
	return this.caseSensitivity;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setRebootCmd(String rebootCmd){
	this.rebootCmd=rebootCmd;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getRebootCmd(){
	return this.rebootCmd;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNoForcedRemove(Boolean noForcedRemove){
	this.noForcedRemove=noForcedRemove;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNoForcedRemove(){
	return this.noForcedRemove;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNoRemove(Boolean noRemove){
	this.noRemove=noRemove;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNoRemove(){
	return this.noRemove;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setSendStatus(Boolean sendStatus){
	this.sendStatus=sendStatus;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getSendStatus(){
	return this.sendStatus;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNoUpgradeBeforeRemove(Boolean noUpgradeBeforeRemove){
	this.noUpgradeBeforeRemove=noUpgradeBeforeRemove;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getNoUpgradeBeforeRemove(){
	return this.noUpgradeBeforeRemove;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogAppend(Boolean logAppend){
	this.logAppend=logAppend;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogAppend(){
	return this.logAppend;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogErrors(Boolean logErrors){
	this.logErrors=logErrors;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogErrors(){
	return this.logErrors;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogWarnings(Boolean logWarnings){
	this.logWarnings=logWarnings;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogWarnings(){
	return this.logWarnings;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogInformation(Boolean logInformation){
	this.logInformation=logInformation;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogInformation(){
	return this.logInformation;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogSuccess(Boolean logSuccess){
	this.logSuccess=logSuccess;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogSuccess(){
	return this.logSuccess;
}

/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogFailure(Boolean logFailure){
	this.logFailure=logFailure;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getLogFailure(){
	return this.logFailure;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogFilePath(String logFilePath){
	this.logFilePath=logFilePath;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getLogFilePath(){
	return this.logFilePath;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setLogFilePattern(String logFilePattern){
	this.logFilePattern=logFilePattern;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getLogFilePattern(){
	return this.logFilePattern;
}
/**
 * Seteja l'identificador de l registre a 0.
 * S'usa per inserir nous registres a la base de dades
 */
public void idNouRegistre(){
	idConfiguracio=0;
}
/**
 * Clona el registre actual
 */
public Object clone(){
    Object obj=null;
    try{
        obj=super.clone();
    }catch(CloneNotSupportedException ex){
        System.out.println(" no se puede duplicar");
    }
    return obj;
}
/**
 * Retorna un String representant el registre actual.
 * Crida el mtode toXml
 * @pre: la instncia actual de l'entitat Configuraci
 * @post: la instncia actual de l'entitat Configuraci
 * 		  String en format xml representant la instncia actual de l'entitat Configuraci
 */
public String toString(){
	return this.toXml();
}

/**
 * Retorna un String XML representant el registre actual.
 * Crida el mtode toDocument
 * @pre: la instncia actual de l'entitat Configuraci
 * @post: la instncia actual de l'entitat Configuraci
 * 		  String en format xml representant la instncia actual de l'entitat Configuraci
 */
public String toXml(){

	String str = "";
	Format format = Format.getPrettyFormat();  
	XMLOutputter xmloutputter = new XMLOutputter(format);  
	str = xmloutputter.outputString(toDocument());  
	return str;
}


/**
 * Retorna un Objecte Document representant el registre actual.
 * @pre: la instncia actual de l'entitat Configuraci
 * @post: la instncia actual de l'entitat Configuraci
 * 		  String en format xml representant la instncia actual de l'entitat Configuraci
 */
public Document toDocument(){

	Document tXml = new Document();  
	Element cArrel = new Element("config");  
	tXml.addContent(cArrel);  
	
	//<param name="forceInstall" value="true"/>
	Element elemForceInstall = new Element("param");
	elemForceInstall.setAttribute("name", "forceInstall");
	elemForceInstall.setAttribute("value", this.forceInstall.toString());
	cArrel.addContent(elemForceInstall);

	//<param name="quitonerror" value="true"/>
	Element elemQuitonerror = new Element("param");
	elemQuitonerror.setAttribute("name", "quitonerror");
	elemQuitonerror.setAttribute("value", this.quitonerror.toString());
	cArrel.addContent(elemQuitonerror);

	//<param name="debug" value="true"/>
	Element elemDebug = new Element("param");
	elemDebug.setAttribute("name", "debug");
	elemDebug.setAttribute("value", this.debug.toString());
	cArrel.addContent(elemDebug);

	//<param name="dryrun" value="false"/>
	Element elemDryrun = new Element("param");
	elemDryrun.setAttribute("name", "dryrun");
	elemDryrun.setAttribute("value", this.dryrun.toString());
	cArrel.addContent(elemDryrun);
	
	//<param name="quiet" value="true"/>
	Element elemQuiet = new Element("param");
	elemQuiet.setAttribute("name", "quiet");
	elemQuiet.setAttribute("value", this.quiet.toString());
	cArrel.addContent(elemQuiet);

	//<param name="nonotify" value="true"/>
	Element elemNonotify = new Element("param");
	elemNonotify.setAttribute("name", "nonotify");
	elemNonotify.setAttribute("value", this.nonotify.toString());
	cArrel.addContent(elemNonotify);

	//<param name="notificationDisplayTime" value="1024"/>
	Element elemNotificationDisplayTime = new Element("param");
	elemNotificationDisplayTime.setAttribute("name", "notificationDisplayTime");
	elemNotificationDisplayTime.setAttribute("value", this.notificationDisplayTime.toString());
	cArrel.addContent(elemNotificationDisplayTime);

	//<param name="noreboot" value="false"/>
	Element elemNoreboot = new Element("param");
	elemNoreboot.setAttribute("name", "noreboot");
	elemNoreboot.setAttribute("value", this.noreboot.toString());
	cArrel.addContent(elemNoreboot);

	//<param name="noRunningState" value="false"/>
	Element elemNoRunningState = new Element("param");
	elemNoRunningState.setAttribute("name", "noRunningState");
	elemNoRunningState.setAttribute("value", this.noRunningState.toString());
	cArrel.addContent(elemNoRunningState);

	//<param name="caseSensitivity" value="true"/>
	Element elemCaseSensitivity = new Element("param");
	elemCaseSensitivity.setAttribute("name", "caseSensitivity");
	elemCaseSensitivity.setAttribute("value", this.caseSensitivity.toString());
	cArrel.addContent(elemCaseSensitivity);

	//<param name="rebootCmd" value="standard2"/>
	Element elemRebootCmd = new Element("param");
	elemRebootCmd.setAttribute("name", "rebootCmd");
	elemRebootCmd.setAttribute("value", this.rebootCmd);
	cArrel.addContent(elemRebootCmd);

	//<param name="noForcedRemove" value="false"/>
	Element elemNoForcedRemove = new Element("param");
	elemNoForcedRemove.setAttribute("name", "noForcedRemove");
	elemNoForcedRemove.setAttribute("value", this.noForcedRemove.toString());
	cArrel.addContent(elemNoForcedRemove);

	//<param name="noRemove" value="false"/>
	Element elemNoRemove = new Element("param");
	elemNoRemove.setAttribute("name", "noRemove");
	elemNoRemove.setAttribute("value", this.noRemove.toString());
	cArrel.addContent(elemNoRemove);

	//<param name="sendStatus" value="false"/>
	Element elemSendStatus = new Element("param");
	elemSendStatus.setAttribute("name", "sendStatus");
	elemSendStatus.setAttribute("value", this.sendStatus.toString());
	cArrel.addContent(elemSendStatus);

	//<param name="noUpgradeBeforeRemove" value="false"/>
	Element elemNoUpgradeBeforeRemove = new Element("param");
	elemNoUpgradeBeforeRemove.setAttribute("name", "noUpgradeBeforeRemove");
	elemNoUpgradeBeforeRemove.setAttribute("value", this.noUpgradeBeforeRemove.toString());
	cArrel.addContent(elemNoUpgradeBeforeRemove);

	//<param name="logAppend" value="false"/>
	Element elemLogAppend = new Element("param");
	elemLogAppend.setAttribute("name", "logAppend");
	elemLogAppend.setAttribute("value", this.logAppend.toString());
	cArrel.addContent(elemLogAppend);

	//<param name="logLevel" value="0xFF"/>
	//*  1: log errors on
	//*  2: log warnings
	//*  4: log information
	//*  8: log audit success
	//* 16: log audit failure

	Integer logLevel = 0;
	
	if (this.logErrors) { logLevel = logLevel + 1;}
	if (this.logWarnings) { logLevel = logLevel + 2;}
	if (this.logInformation) { logLevel = logLevel + 4;}
	if (this.logSuccess) { logLevel = logLevel + 8;}
	if (this.logFailure) { logLevel = logLevel + 16;}

	Element elemLogLevel = new Element("param");
	elemLogLevel.setAttribute("name", "logLevel");
	elemLogLevel.setAttribute("value", logLevel.toString());
	cArrel.addContent(elemLogLevel);

	//<param name="log_file_path" value="%TEMP%3"/>
	Element elemLogFilePath = new Element("param");
	elemLogFilePath.setAttribute("name", "log_file_path");
	elemLogFilePath.setAttribute("value", logFilePath.toString());
	cArrel.addContent(elemLogFilePath);

	//<param name="logfilePattern" value="wpkg-[HOSTNAME].log"/>
	Element elemLogFilePattern = new Element("param");
	elemLogFilePattern.setAttribute("name", "logfilePattern");
	elemLogFilePattern.setAttribute("value", logFilePattern.toString());
	cArrel.addContent(elemLogFilePattern);


	return tXml;
}


}
