package gwpkg;

import java.io.Serializable;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/** Implementa les operacions de la lgica de negoci pertinents a l'entitat configuracio
 */
@Stateful
public class ConfiguracioServeis implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Entity manager JPA usat per les operacions de persistncia de l'entitat configuraci
	 */
	@PersistenceContext(unitName="gwpkg")
	private EntityManager em;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public ConfiguracioServeis(){
	}
	/**
	 * Crea una instncia de configuraci Buida.
	 * Es necessita per donar d'alta nous registres a la base de dades
	 * @post instncia de configuraci buida
	 * @return instncia de configuraci buida
	 */
	public Configuracio configuracioBuida(){
		Configuracio cb = new Configuracio();
		return cb;
	}

	/**
	 * Retorna la llista de configuracions existents a la base de dades.
	 * Filtra o no els resultats per aquelles que estan inhabilitades.
	 * @pre	Existeix mostrarInhabilitats
	 * @post  List<Configuracio> Llista configuracions filtrada
	 * @return  List<Configuracio>
	 */	
	public List<Configuracio> llistaInicial(Boolean mostrarInhabilitats){
		String strQr = "SELECT c FROM Configuracio c";		
		if (!mostrarInhabilitats){
			strQr = strQr + " WHERE c.inhabilitada = :MOSTRAR_INHABILITATS";
		}
		Query qr = em.createQuery(strQr);			
		if (!mostrarInhabilitats){
			qr.setParameter("MOSTRAR_INHABILITATS",mostrarInhabilitats);			
		}
		@SuppressWarnings("unchecked")
		List<Configuracio> llistaInicial = qr.getResultList();
		System.out.println(llistaInicial.size());
		
		return llistaInicial;
	}
	/**
	 * Persisteix una instncia de l'entitat configuraci
	 * @pre	Existeix configuracioActiva
	 * @post  configuracioActiva persistida
	 */	
	public void gravarConfiguracio(Configuracio configuracioActiva){
		em.merge(configuracioActiva);
		}
	/**
	 * Elimina una instncia de l'entitat configuraci
	 * @pre	Existeix configuracioActiva
	 * @post  configuracioActiva eliminada
	 */	
	public void esborrarConfiguracio(Configuracio configuracioActiva){
		em.remove(em.merge(configuracioActiva));
	}
}
