package gwpkg;

import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.ejb.EJB;
import java.io.Serializable;
import java.util.List;
import javax.annotation.PostConstruct;

/**
 * Bean de suport (managed bean) per configuracioPage.xhtml.
 * Cont els atributs i mtodes que implementen el comportament de
 * la interfcie d'usuari.
 */
@SessionScoped
@Named("configuracioView")
public class ConfiguracioView implements Serializable{
	private static final long serialVersionUID = 1L;
	/**
	 * Instncia de l'entitat Configuraci amb la que s'est treballant.
	 */
	@EJB
	private Configuracio configuracioActiva;
	/**
	 * Instncia de l'entitat Configuraci amb la que s'inicialitza un nou registre.
	 */
	private Configuracio configuracioBuida;
	/**
	 * Instncia de l'entitat Configuraci que cont l'ltim estat persistit.
	 * Es fa servir per cancellar les ltimes accions fetes i recuperar l'ltim
	 * estat persistit.
	 */
	private Configuracio configuracioBackUp;
	@EJB
	/**
	 * Implementa la lgica de negoci referent a l'entitat Configuraci.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	private ConfiguracioServeis configuracioServeis;
	/**
	 * Llista contenint les Configuracio disponibles a la base de dades.
	 */
	private List<Configuracio> llistaConfiguracions;
	/**
	 * Apuntador a l'element de la llista de Configuracions disponibles que cont
	 * la configuraci amb la que s'est treballant 
	 */
	private Integer indexLlistaConfiguracions = 0;
	/**
	 * true: A la llista de Configuracions disponibles s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 * false: A la llista de Configuracions disponibles NO s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 */
	private Boolean mostrarInhabilitats=false;
	/**
	 * true: Al inicialitzar un registre nou es clona el registre actiu
	 */
	private Boolean clonarConfiguracio=false;
	/** Flag per renderitzar o no la <div> d'errors a la pgina
	 * false: no es mmostra la <div> d'errors. No s'ha capturat cap excepci.
	 */
	private Boolean displayError=false;
	/**
	 * Mtode que ha capturat la excepci.
	 */
	private String metodeError="";
	/**
	 * Causa de la excepci.(e.getCause().toString();)
	 */
	private String causaError="";
	
	
	
	/**Constructor buit exigit per l'especificaci JSF2
	 */
	public ConfiguracioView(){
	}	

//------------------------------------------------------------
// GETERS I SETERS D'ATRIBUTS JSF2
//------------------------------------------------------------
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getDisplayError(){
		return displayError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDisplayError(Boolean displayError){
		this.displayError=displayError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getMetodeError(){
		return metodeError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMetodeError(String metodeError){
		this.metodeError=metodeError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getCausaError(){
		return causaError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setCausaError(String causaError){
		this.causaError=causaError;
	}
	
	/**Geter exigit per l'especificaci JSF2
	 */
	public Integer getIndexLlistaConfiguracions(){
		return indexLlistaConfiguracions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaConfiguracions(Integer indexLlistaConfiguracions){
		this.indexLlistaConfiguracions=indexLlistaConfiguracions;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Configuracio getConfiguracioActiva(){
		return configuracioActiva;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setConfiguracioActiva(Configuracio configuracioActiva){
		this.configuracioActiva=configuracioActiva;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Configuracio> getLlistaConfiguracions(){
		return llistaConfiguracions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setLlistaConfiguracions(List<Configuracio> llistaConfiguracions){
		this.llistaConfiguracions=llistaConfiguracions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMostrarInhabilitats(Boolean mostrarInhabilitats){
		this.mostrarInhabilitats = mostrarInhabilitats;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getMostrarInhabilitats(){
		return this.mostrarInhabilitats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setClonarConfiguracio(Boolean clonarConfiguracio){
		this.clonarConfiguracio = clonarConfiguracio;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getClonarConfiguracio(){
		return this.clonarConfiguracio;
	}

//------------------------------------------------------------
// MTODES PER IMPLEMENTAR EL COMPORTAMENT DE LA INTERFCIE D'USUARI
//------------------------------------------------------------

	/**
	 * Respn a la petici de l'usuari per Editar la configuraci escollida
	 * @pre: existeix llistaConfiguracions
	 * 		 existeix indexLlistaConfiguracions
	 * @post: configuracioActiva=llistaConfiguracions(indexLlistaConfiguracions) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String carregaConfiguracio(){
		ompleLlistaConfiguracions();
		if (this.llistaConfiguracions.isEmpty()){
			this.configuracioActiva=this.configuracioBuida;	
		} else {
			this.configuracioActiva=this.llistaConfiguracions.get(indexLlistaConfiguracions);
		}
		this.configuracioBackUp = (Configuracio) this.configuracioActiva.clone();
		return "configuracio";
	}
	/**
	 * Respn a la petici de l'usuari per Persistir la configuraci activa
	 * @pre: existeix configuracioActiva
	 * @post: s'ha persistit la configuraci activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarConfiguracio(){		
		try {
			configuracioServeis.gravarConfiguracio(this.getConfiguracioActiva());
			this.configuracioBackUp = (Configuracio) this.configuracioActiva.clone();
			carregaConfiguracio();
		} catch (Exception e) {
			error("gravarConfiguracio",e);
		}
		return "configuracio";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la configuraci activa
	 * @pre: existeix configuracioActiva
	 * @post: s'ha eliminat la configuraci activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String esborrarConfiguracio(){
		try {
			configuracioServeis.esborrarConfiguracio(this.configuracioActiva);
			this.indexLlistaConfiguracions=this.indexLlistaConfiguracions-1;
			if (this.indexLlistaConfiguracions<0){
				this.indexLlistaConfiguracions=this.indexLlistaConfiguracions+1;
			}
			if (llistaConfiguracions.size()>0) {
				} else {
					this.configuracioActiva=this.configuracioBuida;
				}
			carregaConfiguracio();
		} catch (Exception e) {
			error("esborrarConfiguracio",e);
		}
		return "configuracio";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Configuraci
	 * @pre: atribut clonarConfiguracio
	 * @post: si clonarConfiguracio = true
	 * 				clona configuracioActiva amb id = 0 (nou registre)
	 * 		  si clonarConfiguracio = false
	 * 				configuracioActiva = configuracioBuida	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String novaConfiguracio(){
		try {
			if (!clonarConfiguracio) {
				this.configuracioActiva=this.configuracioBuida;
			} else {
				Configuracio novac = (Configuracio) configuracioActiva.clone();
				configuracioActiva = novac;
				configuracioActiva.idNouRegistre();
			}
			this.configuracioBackUp= (Configuracio) this.configuracioActiva.clone();
		} catch (Exception e) {
			error("novaConfiguracio",e);
		}
		return "configuracio";
	}
	
	/**
	 * Respn a la petici de l'usuari per Cancelar les modificacions fetes a Configuracio
	 * desde la ltima vegada que ha estat persistida.
	 * @pre: existeix configuracioBackup
	 *       existeix configuracioActiva
	 * @post: configuracioActiva=configuracioBackup
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String cancelarConfiguracio(){
		try {
			this.configuracioActiva= (Configuracio) this.configuracioBackUp.clone();
		} catch (Exception e) {
			error("cancelarConfiguracio",e);
		}
		return "configuracio";
	}

	/**
	 * Respn a la petici de l'usuari per Filtrar la llista de Configuracions disponibles
	 * segons els criteris especificats.
	 * @pre: existeix llistaConfiguracions
	 *       existeix mostrarInhabilitats
	 * @post: llistaConfiguracions filtrada
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String filtraLlistaConfiguracions(){
		try {
			this.indexLlistaConfiguracions=0;
			ompleLlistaConfiguracions();
		} catch (Exception e) {
			error("filtraLlistaConfiguracions",e);
		}
		return "configuracio";	
	}
	/**
	 * Displaya l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	private void error(String metode,Exception e){
		setDisplayError(true);
		metodeError=metode;

		if (e.getCause()==null){
			causaError=e.getStackTrace()[0].toString();
		} else {
			causaError=e.getCause().toString();
		}
	}

	/**
	 * Esborra el display de l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	public void noDisplayError(){
		setDisplayError(false);
	}
	/**
	 * S'executa per inicialitzar les estructures de la pgina
	 */
	@PostConstruct
	private void ompleLlistaConfiguracions(){
		try {
			this.llistaConfiguracions = configuracioServeis.llistaInicial(mostrarInhabilitats);
			this.configuracioBuida=configuracioServeis.configuracioBuida();
			if (llistaConfiguracions.size()>0) {
				this.configuracioActiva=llistaConfiguracions.get(indexLlistaConfiguracions);
			} else {
				this.configuracioActiva=configuracioBuida;
			}
			this.configuracioBackUp = (Configuracio) this.configuracioActiva.clone();
		} catch (Exception e) {
			error("ompleLlistaConfiguracions",e);
		}
	}
}
