package gwpkg;

import java.io.Serializable;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.*;

/**
 * Classe que representa el model de dades de l'entitat DISTRIBUCIO <P>
 * Una instncia d'aquesta entitat es correspon amb el conjunt de
 * configuracio, hosts i perfils necessaris per generar els fitxers que wpkg.js
 * far servir per la installaci de programari als equips clients
 */
@Entity
@Stateful
@Table(name="distribucio")
public class Distribucio implements Serializable {
	private static final long serialVersionUID = 1L;

	/** Codi identificador de la distribuci. s autonumric. */
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	private int idDistribucio;
	/** Descripci de la Distribucio. Descripci general del contingut d'una distribuci.
	 * Mxim 100 caracters */
	private String desDistribucio;
	/** Estat de la DISTRIBUCIO. Habilitat/Deshabilitat.
	 * Si est habilitat es mostrar a els grids de selecci de Distribucions.
	 * Si est deshabilitat no es mostrar als grids de selecci.
	 * L'estat deshabilitat no comporta cap altre comportament.
	 * BOOLEAN */ 
	private Boolean inhabilitat;
	/** Nom de la Distribucio. Descriptor mnemotcnic d'entre 0 i 15 caracters.*/
	private String nomDistribucio;

	/** bi-directional many-to-one association to Configuracion */
    @ManyToOne
	@JoinColumn(name="idConfiguracio")
	private Configuracio configuracio;

	/** bi-directional many-to-many association to Perfil  */
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(
		name="perfildistribucio"
		, joinColumns={
			@JoinColumn(name="idDistribucio")
			}
		, inverseJoinColumns={
			@JoinColumn(name="idPerfil")
			}
		)
	private List<Perfil> perfils;
	
	/**bi-directional many-to-many association to Host*/
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(
		name="hostdistribucio"
		, joinColumns={
			@JoinColumn(name="idDistribucio")
			}
		, inverseJoinColumns={
			@JoinColumn(name="idHost")
			}
		)
	private List<Host> hosts;

	/** Especifica si s'ha de generar el fitxer config.xml.
	 * BOOLEAN */ 
	private Boolean generarFitxerConfiguracio = true;
	/** Nom del fitxer si no es vol fer servir config.xml.*/ 
	private String nomFitxerConfiguracio = "config.xml";
	/** Directori on es guardar el fitxer config.xml.*/ 
	private String directoriFitxerConfiguracio ;
	

	/** Especifica si s'ha de generar el fitxer packages.xml.
	 * BOOLEAN */ 
	private Boolean generarFitxerPaquets = true;
	/** Nom del fitxer si no es vol fer servir packages.xml.*/ 
	private String nomFitxerPaquets = "packages.xml";
	/** Directori on es guardar el fitxer packages.xml.*/ 
	private String directoriFitxerPaquets ;

	
	/** Especifica si s'ha de generar el fitxer profiles.xml.
	 * BOOLEAN */ 
	private Boolean generarFitxerPerfils = true;
	/** Nom del fitxer si no es vol fer servir profiles.xml.*/ 
	private String nomFitxerPerfils = "profiles.xml";
	/** Directori on es guardar el fitxer profiles.xml.*/ 
	private String directoriFitxerPerfils ;

	
	/** Especifica si s'ha de generar el fitxer hosts.xml.
	 * BOOLEAN */ 
	private Boolean generarFitxerHosts = true;
	/** Nom del fitxer si no es vol fer servir hosts.xml.*/ 
	private String nomFitxerHosts = "hosts.xml";
	/** Directori on es guardar el fitxer hosts.xml.*/ 
	private String directoriFitxerHosts ;
	

	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
    public Distribucio() {
    }

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public int getIdDistribucio() {
		return this.idDistribucio;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setIdDistribucio(int idDistribucio) {
		this.idDistribucio = idDistribucio;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDesDistribucio() {
		return this.desDistribucio;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDesDistribucio(String desDistribucio) {
		this.desDistribucio = desDistribucio;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getInhabilitat() {
		return this.inhabilitat;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setInhabilitat(Boolean inhabilitat) {
		this.inhabilitat = inhabilitat;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomDistribucio() {
		return this.nomDistribucio;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomDistribucio(String nomDistribucio) {
		this.nomDistribucio = nomDistribucio;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Configuracio getConfiguracio() {
		return this.configuracio;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setConfiguracio(Configuracio configuracio) {
		this.configuracio = configuracio;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Perfil> getPerfils() {
		return this.perfils;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setPerfils(List<Perfil> perfils) {
		this.perfils = perfils;
	}
	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Host> getHosts() {
		return this.hosts;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setHosts(List<Host> hosts) {
		this.hosts = hosts;
	}


	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getGenerarFitxerConfiguracio(){
		return generarFitxerConfiguracio;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setGenerarFitxerConfiguracio(Boolean generarFitxerConfiguracio){
		this.generarFitxerConfiguracio=generarFitxerConfiguracio;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomFitxerConfiguracio(){
		return nomFitxerConfiguracio;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomFitxerConfiguracio(String nomFitxerConfiguracio){
		this.nomFitxerConfiguracio=nomFitxerConfiguracio;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDirectoriFitxerConfiguracio(){
		return directoriFitxerConfiguracio;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDirectoriFitxerConfiguracio(String directoriFitxerConfiguracio){
		this.directoriFitxerConfiguracio=directoriFitxerConfiguracio;
	}

	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getGenerarFitxerPaquets(){
		return generarFitxerPaquets;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setGenerarFitxerPaquets(Boolean generarFitxerPaquets){
		this.generarFitxerPaquets=generarFitxerPaquets;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomFitxerPaquets(){
		return nomFitxerPaquets;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomFitxerPaquets(String nomFitxerPaquets){
		this.nomFitxerPaquets=nomFitxerPaquets;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDirectoriFitxerPaquets(){
		return directoriFitxerPaquets;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDirectoriFitxerPaquets(String directoriFitxerPaquets){
		this.directoriFitxerPaquets=directoriFitxerPaquets;
	}
	
	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getGenerarFitxerPerfils(){
		return generarFitxerPerfils;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setGenerarFitxerPerfils(Boolean generarFitxerPerfils){
		this.generarFitxerPerfils=generarFitxerPerfils;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomFitxerPerfils(){
		return nomFitxerPerfils;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomFitxerPerfils(String nomFitxerPerfils){
		this.nomFitxerPerfils=nomFitxerPerfils;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDirectoriFitxerPerfils(){
		return directoriFitxerPerfils;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDirectoriFitxerPerfils(String directoriFitxerPerfils){
		this.directoriFitxerPerfils=directoriFitxerPerfils;
	}

	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getGenerarFitxerHosts(){
		return generarFitxerHosts;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setGenerarFitxerHosts(Boolean generarFitxerHosts){
		this.generarFitxerHosts=generarFitxerHosts;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomFitxerHosts(){
		return nomFitxerHosts;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomFitxerHosts(String nomFitxerHosts){
		this.nomFitxerHosts=nomFitxerHosts;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDirectoriFitxerHosts(){
		return directoriFitxerHosts;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDirectoriFitxerHosts(String directoriFitxerHosts){
		this.directoriFitxerHosts=directoriFitxerHosts;
	}
	
	
	
	/**
	 * Seteja l'identificador del registre a 0.
	 * S'usa per inserir nous registres a la base de dades
	 */
	public void idNouRegistre(){
		idDistribucio=0;
	}
	
}