package gwpkg;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.jdom.Document;  
import org.jdom.Element;  
import org.jdom.output.Format; 
import org.jdom.output.XMLOutputter;

/** Implementa les operacions de la lgica de negoci pertinents a l'entitat Distribucio
 */
@Stateful
public class DistribucioServeis implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Entity manager JPA usat per les operacions de persistncia de l'entitat Distribuico
	 */
	@PersistenceContext(unitName="gwpkg")
	private EntityManager em;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public DistribucioServeis(){
	}
	
	/**
	 * Crea una instncia de Distribucio Buida.
	 * Es necessita per donar d'alta nous registres a la base de dades
	 * @post instncia de Distribucio buida
	 * @return instncia de Distribucio buida
	 */
	public Distribucio distribucioBuit(){
		Distribucio d = new Distribucio();
		return d;
	}
	/**
	 * Clona el registre actual
	 */
	public Distribucio clonarDistribucio(Distribucio distribucioOriginal){
		
		Distribucio distribucioClonat = distribucioBuit();
	
		distribucioClonat.setIdDistribucio(distribucioOriginal.getIdDistribucio());
		distribucioClonat.setInhabilitat(distribucioOriginal.getInhabilitat());
		distribucioClonat.setNomDistribucio(distribucioOriginal.getNomDistribucio());
		distribucioClonat.setDesDistribucio(distribucioOriginal.getDesDistribucio());

		distribucioClonat.setGenerarFitxerConfiguracio(distribucioOriginal.getGenerarFitxerConfiguracio());
		distribucioClonat.setNomFitxerConfiguracio(distribucioOriginal.getNomFitxerConfiguracio());		
		distribucioClonat.setDirectoriFitxerConfiguracio(distribucioOriginal.getDirectoriFitxerConfiguracio());		

		distribucioClonat.setGenerarFitxerPaquets(distribucioOriginal.getGenerarFitxerPaquets());
		distribucioClonat.setNomFitxerPaquets(distribucioOriginal.getNomFitxerPaquets());		
		distribucioClonat.setDirectoriFitxerPaquets(distribucioOriginal.getDirectoriFitxerPaquets());		
		
		distribucioClonat.setGenerarFitxerPerfils(distribucioOriginal.getGenerarFitxerPerfils());
		distribucioClonat.setNomFitxerPerfils(distribucioOriginal.getNomFitxerPerfils());		
		distribucioClonat.setDirectoriFitxerPerfils(distribucioOriginal.getDirectoriFitxerPerfils());		

		distribucioClonat.setGenerarFitxerHosts(distribucioOriginal.getGenerarFitxerHosts());
		distribucioClonat.setNomFitxerHosts(distribucioOriginal.getNomFitxerHosts());		
		distribucioClonat.setDirectoriFitxerHosts(distribucioOriginal.getDirectoriFitxerHosts());		
		
		if (distribucioOriginal.getConfiguracio() != null){
			distribucioClonat.setConfiguracio(distribucioOriginal.getConfiguracio());
		}
		List<Perfil> l = new ArrayList<Perfil>();
		distribucioClonat.setPerfils(l);
		if (distribucioOriginal.getPerfils()!= null){
			distribucioClonat.getPerfils().addAll(distribucioOriginal.getPerfils());
		}
		List<Host> h = new ArrayList<Host>();
		distribucioClonat.setHosts(h);
		if (distribucioOriginal.getHosts()!= null){
			distribucioClonat.getHosts().addAll(distribucioOriginal.getHosts());
		}
		return distribucioClonat;
	}
	
	/**
	 * Retorna la llista de Distribucions existents a la base de dades.
	 * Filtra o no els resultats per aquells que estan inhabilitats.
	 * @pre	Existeix mostrarInhabilitats
	 * @post  List<Distribucio> Llista Distribucions filtrada
	 * @return  List<Distribucio>
	 */	
	public List<Distribucio> llista(Boolean inhabilitats){
		String strQr = "SELECT d FROM Distribucio d";		
		if (!inhabilitats){
			strQr = strQr + " WHERE d.inhabilitat = :MOSTRAR_INHABILITATS";
		}
		Query qr = em.createQuery(strQr);			
		if (!inhabilitats){
			qr.setParameter("MOSTRAR_INHABILITATS",inhabilitats);			
		}
		@SuppressWarnings("unchecked")
		List<Distribucio> llistaInicial = qr.getResultList();
		
		return llistaInicial;
	}
	
	/**
	 * Persisteix una instncia de l'entitat Distribucio
	 * @pre	Existeix distribucioActiu
	 * @post  distribucioActiu persistit
	 */	
	public void gravarDistribucio(Distribucio distribucioActiu){
			em.merge(distribucioActiu);
	}
	/**
	 * Elimina una instncia de l'entitat Distribucio
	 * @pre	Existeix distribucioActiu
	 * @post  distribucioActiu eliminat
	 */	
	public void esborrarDistribucio(Distribucio distribucioActiu){
		em.remove(em.merge(distribucioActiu));
		}

	/**
	 * Genera el fitxer config.xml i l'escriu a disc. No te en conta el parmetre generarFitxerConfiguracio
	 * @pre	Existeix distribucioActiu
	 * @post  fitxer xml generat
	 */	
	public void configuracioToXml(Distribucio distribucioActiu){
		Document tXml = new Document();
	
		tXml=distribucioActiu.getConfiguracio().toDocument();
		grabarXml(distribucioActiu.getDirectoriFitxerConfiguracio(),distribucioActiu.getNomFitxerConfiguracio(),tXml);
		}

	/**
	 * Genera el fitxer packets.xml i l'escriu a disc. No te en conta el parmetre generarFitxerPaquets
	 * @pre	Existeix distribucioActiu
	 * @post  fitxer xml generat
	 */	
	public void paquetsToXml(Distribucio distribucioActiu){

		Document tXml = new Document();  
		Element paquetsArrel = new Element("packages");  
		tXml.addContent(paquetsArrel);
		
		List<Paquet> paquets = new ArrayList<Paquet>();
		List<Perfil> perfils = distribucioActiu.getPerfils();
		List<Perfil> perfilsXml =  new ArrayList<Perfil>();

		perfilsXml.addAll(perfils);
		/* Afegeix a la llista de perfils de la Distribucio els perfils dels
		   que depen cada un d'ells de manera que el fixer packets.xml els 
		   contingui tamb 		
		*/
		Iterator<Perfil> itProfiles = perfils.iterator();
		while (itProfiles.hasNext()){
			Perfil perfil = (Perfil) itProfiles.next();
			List<Perfil> perfilsDepen = perfil.getPerfilsDepen();
			perfilsXml.addAll(perfilsDepen);
		}
		
		Iterator<Perfil> itProfilesXml = perfilsXml.iterator();
		while (itProfilesXml.hasNext()) {
			Perfil perfil = (Perfil) itProfilesXml.next();
			List<Paquet> paquetsPerfil = perfil.getPaquets();
			Iterator<Paquet> itPaquets = paquetsPerfil.iterator();
			while (itPaquets.hasNext()) {
				Paquet paquet = (Paquet) itPaquets.next();
				if (!paquets.contains(paquet)){
					paquets.add(paquet);
					Element elemPaquet = new Element("package");
					elemPaquet.setAttribute("id", paquet.getNomPaquet());
					elemPaquet.setAttribute("name", paquet.getDescripcioPaquet());
					elemPaquet.setAttribute("revision", paquet.getRevisio().toString());
					elemPaquet.setAttribute("reboot", paquet.getReiniciar().toString());
					elemPaquet.setAttribute("priority", paquet.getPrioritat().toString());

					List<Comprovacio> comprovacionsPaquet=paquet.getComprovacionsPaquet();
					Iterator<Comprovacio> itComprovacions = comprovacionsPaquet.iterator();
					while (itComprovacions.hasNext()) {
						Comprovacio comprovacio = (Comprovacio) itComprovacions.next();
						Element elemComprovacio = new Element("check");
						elemComprovacio.setAttribute("cmd", comprovacio.getDescripcioComprovacio());
						elemPaquet.addContent(elemComprovacio);
					}

					List<Instalacio> instalacionsPaquet=paquet.getInstalacionsPaquet();
					Iterator<Instalacio> itInstalacions = instalacionsPaquet.iterator();
					while (itInstalacions.hasNext()) {
						Instalacio instalacio = (Instalacio) itInstalacions.next();
						Element elemInstalacio = new Element("install");
						elemInstalacio.setAttribute("cmd", instalacio.getDescripcioInstalacio());
						elemPaquet.addContent(elemInstalacio);
					}

					List<Actualitzacio> actualitzacionsPaquet=paquet.getActualitzacionsPaquet();
					Iterator<Actualitzacio> itActualitzacions = actualitzacionsPaquet.iterator();
					while (itActualitzacions.hasNext()) {
						Actualitzacio actualitzacio = (Actualitzacio) itActualitzacions.next();
						Element elemActualitzacio = new Element("upgrade");
						elemActualitzacio.setAttribute("cmd", actualitzacio.getDescripcioActualitzacio());
						elemPaquet.addContent(elemActualitzacio);
					}

					List<Desinstalacio> desinstalacionsPaquet=paquet.getDesinstalacionsPaquet();
					Iterator<Desinstalacio> itDesinstalacions = desinstalacionsPaquet.iterator();
					while (itDesinstalacions.hasNext()) {
						Desinstalacio desinstalacio = (Desinstalacio) itDesinstalacions.next();
						Element elemDesinstalacio = new Element("remove");
						elemDesinstalacio.setAttribute("cmd", desinstalacio.getDescripcioDesinstalacio());
						elemPaquet.addContent(elemDesinstalacio);
					}

					paquetsArrel.addContent(elemPaquet);
				}			
			}
		}
		  
		grabarXml(distribucioActiu.getDirectoriFitxerPaquets(),distribucioActiu.getNomFitxerPaquets(),tXml);
	}

	/**
	 * Genera el fitxer profiles.xml i l'escriu a disc. No te en conta el parmetre generarFitxerPerfils
	 * @pre	Existeix distribucioActiu
	 * @post  fitxer xml generat
	 */	
	public void perfilsToXml(Distribucio distribucioActiu){
		Document tXml = new Document();  
		Element perfilsArrel = new Element("profiles");  
		tXml.addContent(perfilsArrel);

		List<Perfil> perfils = distribucioActiu.getPerfils();		
		Iterator<Perfil> itPerfils = perfils.iterator();
		while (itPerfils.hasNext()) {
			Perfil perfil = itPerfils.next();
			Element elemPerfil = new Element("profile");
			elemPerfil.setAttribute("id", perfil.getNomPerfil());

			List<Paquet> paquets = perfil.getPaquets();
			Iterator<Paquet> itPaquets = paquets.iterator();
			while (itPaquets.hasNext()) {
				Paquet paquet = (Paquet) itPaquets.next();
				Element elemPaquet = new Element("package");
				elemPaquet.setAttribute("id", paquet.getNomPaquet());
				elemPerfil.addContent(elemPaquet);
			}		
			perfilsArrel.addContent(elemPerfil);
		}
		grabarXml(distribucioActiu.getDirectoriFitxerPerfils(),distribucioActiu.getNomFitxerPerfils(),tXml);
	}

	/**
	 * Genera el fitxer hosta.xml i l'escriu a disc. No te en conta el parmetre generarFitxerHosts
	 * @pre	Existeix distribucioActiu
	 * @post  fitxer xml generat
	 */	
	public void hostsToXml(Distribucio distribucioActiu){
		Document tXml = new Document();  
		Element hostsArrel = new Element("wpkg");  
		tXml.addContent(hostsArrel);

		List<Host> hosts = distribucioActiu.getHosts();		
		Iterator<Host> itHosts = hosts.iterator();
		while (itHosts.hasNext()) {
			Host host = itHosts.next();
			Element elemHost = new Element("host");
			elemHost.setAttribute("name", host.getNomHost());
	
			List<Perfil> perfils = host.getPerfils();
			Iterator<Perfil> itPerfils = perfils.iterator();
			while (itPerfils.hasNext()) {
				Perfil perfil = (Perfil) itPerfils.next();
				if (perfils.indexOf(perfil)==0){
					elemHost.setAttribute("profile-id", perfil.getNomPerfil());
				} else {
					Element elemPerfil = new Element("profile");
					elemPerfil.setAttribute("id", perfil.getNomPerfil());
					elemHost.addContent(elemPerfil);
				}
			}		
			hostsArrel.addContent(elemHost);			
		}

		grabarXml(distribucioActiu.getDirectoriFitxerHosts(),distribucioActiu.getNomFitxerHosts(),tXml);
	}
	
	/**
	 * Mtode auxiliar. Grava a disc un fitxer Xml a partir d'un objecte Document. Si no existeix el directori el crea.
	 * @param Directori on es grabar el fitxer
	 * @param Nom del fitxer
	 * @param Document representant el xml.
	 * @post  fitxer xml generat
	 */	
	private void grabarXml(String lloc,String nom, Document sortida){

		try {

			Format format = Format.getPrettyFormat();  
			XMLOutputter xmloutputter = new XMLOutputter(format);  
			String xml = xmloutputter.outputString(sortida);
			File fitxer = new File(lloc);
			if (!fitxer.exists()){
				fitxer.mkdir();
			}
			lloc = lloc + "\\" + nom;
			BufferedWriter out = new BufferedWriter(new FileWriter(lloc));
			out.write(xml);
			out.close();

		}catch (IOException e){
			System.out.println(e.toString());
		}

	}
}
