package gwpkg;

import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.ejb.EJB;
import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;
import javax.annotation.PostConstruct;

/**
 * Bean de suport (managed bean) per hostPage.xhtml.
 * Cont els atributs i mtodes que implementen el comportament de
 * la interfcie d'usuari.
 */
@SessionScoped
@Named("distribucioView")
public class DistribucioView implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Implementa la lgica de negoci referent a l'entitat Distribucio.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private DistribucioServeis distribucioServeis;
	/**
	 * Instncia de l'entitat Host amb la que s'est treballant.
	 */
	@EJB
	private Distribucio distribucioActiu;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Perfil.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private PerfilServeis perfilServeis;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Configuracio.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private ConfiguracioServeis configuracioServeis;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Host.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private HostServeis hostServeis;
	
	/**
	 * Instncia de l'entitat Distribucio amb la que s'inicialitza un nou registre.
	 */
	private Distribucio distribucioBuit;
	/**
	 * Instncia de l'entitat Distribucio que cont l'ltim estat persistit.
	 * Es fa servir per cancellar les ltimes accions fetes i recuperar l'ltim
	 * estat persistit.
	 */
	private Distribucio distribucioBackUp;
	
	/**
	 * Apuntador a l'element de la llista de Distribucions que cont
	 * la base de dades 
	 */
	private Integer indexLlistaDistribucions;
	/**
	 * Apuntador a l'element actiu de la llista de Perfils assignats
	 */
	private Integer indexLlistaPerfilsAssignats;
	/**
	 * Apuntador a l'element actiu de la llista de Hosts assignats
	 */
	private Integer indexLlistaHostsAssignats;
	
	/**
	 * Apuntador a l'element actiu de la llista de Perfils disponibles
	 */
	private Integer indexLlistaPerfilsDisponibles;
	/**
	 * Apuntador a l'element actiu de la llista de Configuracions disponibles
	 */
	private Integer indexLlistaConfiguracionsDisponibles;
	/**
	 * Apuntador a l'element actiu de la llista de Hosts disponibles
	 */
	private Integer indexLlistaHostsDisponibles;
	
	/**
	 * Llista contenint les Distribucions disponibles a la base de dades.
	 */
	private List<Distribucio> llistaDistribucions;
	/**
	 * Llista contenint els Perfils disponibles a la base de dades.
	 */
	private List<Perfil> perfilsDisponibles;
	/**
	 * Llista contenint les Configuracions disponibles a la base de dades.
	 */
	private List<Configuracio> configuracionsDisponibles;
	/**
	 * Llista contenint els Hosts disponibles a la base de dades.
	 */
	private List<Host> hostsDisponibles;
	
	/**
	 * true: A la llista de Distribucions disponibles s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 * false: A la llista de Distribucions disponibles NO s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 */
	private Boolean mostrarInhabilitats=false;
	/**
	 * true: Al inicialitzar un registre nou es clona el registre actiu
	 */
	private Boolean clonarDistribucio=false;
	/** Nom de la configuracio assignada a aquesta distribucio.
	 */
	private String nomConfiguracioAssignada="";

	/** Flag per renderitzar o no la <div> d'errors a la pgina
	 * false: no es mmostra la <div> d'errors. No s'ha capturat cap excepci.
	 */
	private Boolean displayError=false;
	/**
	 * Mtode que ha capturat la excepci.
	 */
	private String metodeError="";
	/**
	 * Causa de la excepci.(e.getCause().toString();)
	 */
	private String causaError="";

	
	/**Constructor buit exigit per l'especificaci JSF2
	 */
	public DistribucioView(){
	}

	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getDisplayError(){
		return displayError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDisplayError(Boolean displayError){
		this.displayError=displayError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getMetodeError(){
		return metodeError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMetodeError(String metodeError){
		this.metodeError=metodeError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getCausaError(){
		return causaError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setCausaError(String causaError){
		this.causaError=causaError;
	}
	
	// -------- DISTRIBUCIO GETERS I SETTERS
	/**Geter exigit per l'especificaci JSF2
	 */
	public Integer getIndexLlistaDistribucions(){
		return indexLlistaDistribucions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaDistribucions(Integer indexLlistaDistribucions){
		this.indexLlistaDistribucions=indexLlistaDistribucions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsAssignats(Integer indexLlistaPerfilsAssignats){
		this.indexLlistaPerfilsAssignats=indexLlistaPerfilsAssignats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsDisponibles(Integer indexLlistaPerfilsDisponibles){
		this.indexLlistaPerfilsDisponibles=indexLlistaPerfilsDisponibles;		
	}

	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaConfiguracionsDisponibles(Integer indexLlistaConfiguracionsDisponibles){
		this.indexLlistaConfiguracionsDisponibles=indexLlistaConfiguracionsDisponibles;		
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaHostsDisponibles(Integer indexLlistaHostsDisponibles){
		this.indexLlistaHostsDisponibles=indexLlistaHostsDisponibles;		
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaHostsAssignats(Integer indexLlistaHostsAssignats){
		this.indexLlistaHostsAssignats=indexLlistaHostsAssignats;		
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Distribucio getDistribucioActiu(){
		return distribucioActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDistribucioActiu(Distribucio distribucioActiu){
		this.distribucioActiu=distribucioActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDistribucioBuit(Distribucio distribucioBuit){
		this.distribucioBuit=distribucioBuit;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Distribucio getDistribucioBuit(){
		return distribucioBuit;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDistribucioBackUp(Distribucio distribucioBackUp){
		this.distribucioBackUp=distribucioBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Distribucio getDistribucioBackUp(){
		return distribucioBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Distribucio> getLlistaDistribucions(){
		return llistaDistribucions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setLlistaDistribucions(List<Distribucio> llistaDistribucions){
		this.llistaDistribucions=llistaDistribucions;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMostrarInhabilitats(Boolean mostrarInhabilitats){
		this.mostrarInhabilitats = mostrarInhabilitats;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getMostrarInhabilitats(){
		return this.mostrarInhabilitats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setClonarDistribucio(Boolean clonarDistribucio){
		this.clonarDistribucio = clonarDistribucio;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getClonarDistribucio(){
		return this.clonarDistribucio;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setNomConfiguracioAssignada(String nomConfiguracioAssignada){
		this.nomConfiguracioAssignada=nomConfiguracioAssignada;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getNomConfiguracioAssignada(){
		return nomConfiguracioAssignada;
	}
	
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilsDisponibles(){
		perfilsDisponibles = perfilServeis.llista(false);	
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Perfil> getPerfilsDisponibles(){
		return perfilsDisponibles;
	}
	
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setConfiguracionsDisponibles(){
		configuracionsDisponibles = configuracioServeis.llistaInicial(false);	
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Configuracio> getConfiguracionsDisponibles(){
		return configuracionsDisponibles;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setHostsDisponibles(){
		hostsDisponibles = hostServeis.llista(false);	
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Host> getHostsDisponibles(){
		return hostsDisponibles;
	}
	
	//--------------- DISTRIBUCIO ACCIONS
	
	/**
	 * Respn a la petici de l'usuari per Editar la Distribucio escollida
	 * @pre: existeix llistaDistribucions
	 * 		 existeix indexLlistaDistribucions
	 * @post: distriubucioActiu=llistaDistribucions(indexLlistaDistribucions) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarDistribucio(){
		try {
			canviaDistribucioActiu(getIndexLlistaDistribucions());		
		} catch	(Exception e) {
			error("editarDistribucio",e);
		}	
		return "distribucio";
	}
	
	/**
	 * Respn a la petici de l'usuari per Editar la Distribucio escollida
	 * @pre: existeix llistaDistribucions
	 * 		 existeix indexLlistaDistribucions
	 * @post: distriubucioActiu=llistaDistribucions(indexLlistaDistribucions) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public void canviaDistribucioActiu(Integer indexLlistaDistribucions){
		if (indexLlistaDistribucions >= 0) {
			setIndexLlistaDistribucions(indexLlistaDistribucions);
			setDistribucioActiu(getLlistaDistribucions().get(indexLlistaDistribucions));
		} else {
			setIndexLlistaDistribucions(0);
			setDistribucioActiu(getDistribucioBuit());
		}
		setDistribucioBackUp(distribucioServeis.clonarDistribucio(distribucioActiu));
		if (getDistribucioActiu().getConfiguracio() != null) {
			setNomConfiguracioAssignada(getDistribucioActiu().getConfiguracio().getNomConfiguracio());
		} else {
			setNomConfiguracioAssignada("");
		}
}

	/**
	 * Respn a la petici de l'usuari per Persistir la Distribucio activa
	 * @pre: existeix distribucioActiva
	 * @post: s'ha persistit  LA distribucioActiva
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarDistribucio(){
		try {
			distribucioServeis.gravarDistribucio(getDistribucioActiu());
			if (getDistribucioActiu().getIdDistribucio()==0) {
				ompleLlistaDistribucions();
			}
		} catch (Exception e) {
			error("gravarDistribucio",e);
		}
		return "distribucio";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la Distribucio activa
	 * @pre: existeix distribucioActiva
	 * @post: s'ha eliminat LA distribucioActiva
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String esborrarDistribucio(){
		try {
			distribucioServeis.esborrarDistribucio(getDistribucioActiu());
			getLlistaDistribucions().remove(getDistribucioActiu());
			setIndexLlistaDistribucions(getIndexLlistaDistribucions()-1);
			canviaDistribucioActiu(getIndexLlistaDistribucions());
		} catch (Exception e) {
			error("esborrarDistribucio",e);
		}
		return "distribucio";			
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Distribucio
	 * @pre: atribut clonarDistribucio
	 * @post: si clonarDistribucio = true
	 * 				clona distribucioActiu amb id = 0 (nou registre)
	 * 		  si clonarDistribucio = false
	 * 				distribucioActiu = distribucioBuit	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String nouDistribucio(){
		try {
			if (!clonarDistribucio) {
				setDistribucioActiu(distribucioServeis.distribucioBuit());
				setNomConfiguracioAssignada("");
			} else {
				Distribucio noud = distribucioServeis.clonarDistribucio(distribucioActiu);
				setDistribucioActiu(noud);
				getDistribucioActiu().idNouRegistre();
			}		
			setDistribucioBackUp(distribucioServeis.clonarDistribucio(distribucioActiu));
		} catch (Exception e) {
			error("nouDistribucio",e);
		}	
		return "distribucio";
	}
	
	/**
	 * Respn a la petici de l'usuari per Cancelar les modificacions fetes a Distribucio
	 * desde la ltima vegada que ha estat persistida.
	 * @pre: existeix distribucioBackup
	 *       existeix distribucioActiu
	 * @post: distribucioActiu=distribucioBackup
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String cancelarDistribucio(){
		try {
			setDistribucioActiu( distribucioServeis.clonarDistribucio(distribucioBackUp));
			getLlistaDistribucions().set(getIndexLlistaDistribucions(), getDistribucioActiu());
			if (getDistribucioActiu().getConfiguracio() != null) {
				setNomConfiguracioAssignada(getDistribucioActiu().getConfiguracio().getNomConfiguracio());
			} else {
				setNomConfiguracioAssignada("");
			}
		} catch	(Exception e) {
			error("cancelarDistribucio",e);
		}	
		return "distribucio";
	}
		
	/**
	 * Respn a la petici de l'usuari per Filtrar la llista de Distribucions disponibles
	 * segons els criteris especificats.
	 * @pre: existeix llistaDistribucions
	 *       existeix mostrarInhabilitats
	 * @post: llistaDistribucions filtrada
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String filtraLlistaDistribucions(){
		try {
			ompleLlistaDistribucions();
		} catch	(Exception e) {
			error("filtraLlistaDistribucions",e);
		}	
		return "distribucio";	
	}
	/**
	 * Respn a la petici de l'usuari per assignar una Configuracio a la Distribucio activa
	 * @pre: existeix distribucioActiva
	 *       existeix indexLlistaConfiguracionsDisponibles
	 * @post: paquet  indexLlistaConfiguracionsDisponibles assignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String assignarConfiguracio(){
		try {
			distribucioActiu.setConfiguracio(configuracionsDisponibles.get(indexLlistaConfiguracionsDisponibles));
			setNomConfiguracioAssignada(getDistribucioActiu().getConfiguracio().getNomConfiguracio());
		} catch	(Exception e) {
			error("assignarConfiguracio",e);
		}	
		return "distribucio";
	}	
	/**
	 * Respn a la petici de l'usuari per deassignar un perfil a la Distribucio activa
	 * @pre: existeix distribucioActiva
	 *       existeix indexLlistaPerfilsAssignats
	 * @post: paquet  indexLlistaPerfilsAssignats desassignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String desassignarPerfil(){
		try {
			List<Perfil> llistaPerfils = distribucioActiu.getPerfils();
			Perfil perfil = llistaPerfils.get(indexLlistaPerfilsAssignats);
			llistaPerfils.remove(perfil);
		} catch	(Exception e) {
			error("desassignarPerfil",e);
		}	
		return "distribucio";
	}
	/**
	 * Respn a la petici de l'usuari per assignar un Perfil a la Distribucio activa
	 * @pre: existeix distribucioActiva
	 *       existeix indexLlistaPerfilsDisponibles
	 * @post: paquet  indexLlistaPerfilsDisponibles assignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String assignarPerfil(){
		try {
			Perfil perfil = perfilsDisponibles.get(indexLlistaPerfilsDisponibles);
			if (distribucioActiu.getPerfils() != null) {
				List<Perfil> llistaPerfils = distribucioActiu.getPerfils();
				if (!(llistaPerfils.contains(perfil))) {
					llistaPerfils.add(perfil);
				}
			} else {
				List<Perfil> llistaPerfils = new ArrayList<Perfil>();	
				distribucioActiu.setPerfils(llistaPerfils);			
				llistaPerfils.add(perfil);
			}

		} catch	(Exception e) {
			error("assignarPerfil",e);
		}	
		return "distribucio";
	}
	/**
	 * Respn a la petici de l'usuari per deassignar un Host a la Distribucio activa
	 * @pre: existeix distribucioActiva
	 *       existeix indexLlistaHostsAssignats
	 * @post: paquet  indexLlistaHostsAssignats desassignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String desassignarHost(){
		try {
			List<Host> llistaHosts = distribucioActiu.getHosts();
			Host host = llistaHosts.get(indexLlistaHostsAssignats);
			llistaHosts.remove(host);
		} catch	(Exception e) {
			error("desassignarHost",e);
		}	
		return "distribucio";
	}
	/**
	 * Respn a la petici de l'usuari per assignar un Host a la Distribucio activa
	 * @pre: existeix distribucioActiva
	 *       existeix indexLlistaHotsDisponibles
	 * @post: paquet  indexLlistaHostsDisponibles assignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String assignarHost(){
		try {
			Host host = hostsDisponibles.get(indexLlistaHostsDisponibles);
			if (distribucioActiu.getHosts() != null) {
				List<Host> llistaHosts = distribucioActiu.getHosts();
				if (!(llistaHosts.contains(host))) {
					llistaHosts.add(host);
				}
			} else {
				List<Host> llistaHosts = new ArrayList<Host>();	
				distribucioActiu.setHosts(llistaHosts);			
				llistaHosts.add(host);
			}

		} catch	(Exception e) {
			error("assignarHost",e);
		}	
		return "distribucio";
	}

	/**
	 * Respn a la petici de l'usuari per crear el fitxer config.xml
	 * No te en conta el parmetre generarFitxerConfiguracio
	 * @pre: existeix distribucioActiva
	 * @post: config.xml creat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String generarXmlConfiguracio(){
		try {
			distribucioServeis.configuracioToXml(distribucioActiu);
		}catch (Exception e){
			error("generarXmlConfiguracio",e);
		}	
		return "distribucio";	
	}
	/**
	 * Respn a la petici de l'usuari per crear el fitxer packets.xml
	 * No te en conta el parmetre generarFitxerPaquets
	 * @pre: existeix distribucioActiva
	 * @post: packets.xml creat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String generarXmlPaquets(){
		try {
			distribucioServeis.paquetsToXml(distribucioActiu);
		}catch (Exception e){
			error("generarXmlPaquets",e);
		}	
		return "distribucio";	
	}
	/**
	 * Respn a la petici de l'usuari per crear el fitxer perfils.xml
	 * No te en conta el parmetre generarFitxerPerfils
	 * @pre: existeix distribucioActiva
	 * @post: perfils.xml creat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String generarXmlPerfils(){
		try {
			distribucioServeis.perfilsToXml(distribucioActiu);
		}catch (Exception e){
			error("generarXmlPerfils",e);
		}	
		return "distribucio";	
	}
	/**
	 * Respn a la petici de l'usuari per crear el fitxer hosts.xml
	 * No te en conta el parmetre generarFitxerHosts
	 * @pre: existeix distribucioActiva
	 * @post: hosts.xml creat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String generarXmlHosts(){
		try {
			distribucioServeis.hostsToXml(distribucioActiu);
		}catch (Exception e){
			error("generarXmlHosts",e);
		}	
		return "distribucio";	
	}
	/**
	 * Respn a la petici de l'usuari per generar tots els fitxers .xml de la distribucio
	 * Te en conta els parmetres generarFitxerXXXX
	 * @pre: existeix distribucioActiva
	 * @post: fitxers .xml creats
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String generarXmlTots(){
		if (distribucioActiu.getGenerarFitxerPaquets()==true) {
				distribucioServeis.configuracioToXml(distribucioActiu);
		}
		if (distribucioActiu.getGenerarFitxerPaquets()==true) {
				distribucioServeis.paquetsToXml(distribucioActiu);
		}
		if (distribucioActiu.getGenerarFitxerPerfils()==true) {
				distribucioServeis.perfilsToXml(distribucioActiu);
		}
		if (distribucioActiu.getGenerarFitxerHosts()==true) {
				distribucioServeis.hostsToXml(distribucioActiu);
		}
		return "distribucio";	
	}
	/**
	 * Displaya l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	private void error(String metode,Exception e){
		setDisplayError(true);
		metodeError=metode;
		if (e.getCause()==null){
			causaError=e.getStackTrace()[0].toString();
		} else {
			causaError=e.getCause().toString();
		}
	}
	/**
	 * Esborra el display de l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	public void noDisplayError(){
		setDisplayError(false);
	}
	
	//------------- POSTCONSTRUCT
	/**
	 * S'executa per inicialitzar les estructures de la pgina
	 */		
	@PostConstruct
	private void ompleLlistaDistribucions(){
		setLlistaDistribucions(distribucioServeis.llista(getMostrarInhabilitats()));
		setDistribucioBuit(distribucioServeis.distribucioBuit());
		setIndexLlistaDistribucions(0);
		if (getLlistaDistribucions().size()>0) {
			setDistribucioActiu(getLlistaDistribucions().get(getIndexLlistaDistribucions()));
		} else {
			setDistribucioActiu(getDistribucioBuit());
		}
		setDistribucioBackUp(distribucioServeis.clonarDistribucio(distribucioActiu));
		setConfiguracionsDisponibles();
		if (getDistribucioActiu().getConfiguracio()!=null) {
			setNomConfiguracioAssignada(getDistribucioActiu().getConfiguracio().getNomConfiguracio());
		} else {
			setNomConfiguracioAssignada("");
		}
		setPerfilsDisponibles();
		setHostsDisponibles();		
	}
}
