package gwpkg;

import java.io.Serializable;
import javax.persistence.*;

import java.util.ArrayList;
import java.util.List;
import javax.ejb.Stateful;

/**
 * Classe que representa el model de dades de l'entitat HOST <P>
 * Una instncia d'aquesta entitat es correspon amb un equip
 * (ordinador)del que es preveu gestionar la installaci
 * desinstallaci de software amb el paquet WPKG  
 */
@Entity
@Stateful
@Table(name="host")
public class Host implements Serializable {
	private static final long serialVersionUID = 1L;

	/** Codi identificador del Host. s autonumric. */
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	private int idHost;
	/** Descripci del Host. Descripci general del contingut d'un perfil.
	 * Mxim 100 caracters */
	private String descripcioHost;
	/** Nom del Host. Descriptor mnemotcnic d'entre 0 i 15 caracters.*/
	private String nomHost;
	/** Estat del HOST. Habilitat/Deshabilitat.
	 * Si est habilitat es mostrar a els grids de selecci de Hosts.
	 * Si est deshabilitat no es mostrar als grids de selecci.
	 * L'estat deshabilitat no comporta cap altre comportament.
	 * BOOLEAN */ 
	private Boolean inhabilitat;

	/** bi-directional many-to-many association to Perfil */
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(
		name="perfilhost"
		, joinColumns={
			@JoinColumn(name="idhost")
			}
		, inverseJoinColumns={
			@JoinColumn(name="idperfil")
			}
		)
	private List<Perfil> perfils = new ArrayList<Perfil>();
	
	/** bi-directional many-to-many association to Distribucio */
	@ManyToMany(mappedBy="hosts", fetch=FetchType.EAGER)
	private List<Distribucio> distribucions;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
    public Host() {
    }

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public int getIdHost() {
		return this.idHost;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setIdHost(int idHost) {
		this.idHost = idHost;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDescripcioHost() {
		return this.descripcioHost;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDescripcioHost(String descripcioHost) {
		this.descripcioHost = descripcioHost;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomHost() {
		return this.nomHost;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomHost(String nomHost) {
		this.nomHost = nomHost;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getInhabilitat(){
		return this.inhabilitat;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setInhabilitat(Boolean inhabilitat){
		this.inhabilitat=inhabilitat;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Perfil> getPerfils() {
		return this.perfils;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setPerfils(List<Perfil> perfils) {
		this.perfils = perfils;
	}
	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Distribucio> getDistribucions() {
		return this.distribucions;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDistribucions(List<Distribucio> distribucions) {
		this.distribucions = distribucions;
	}
	
	/**
	 * Clona el registre actual
	 */
	public Object clone(){
	    Object obj=null;
	    try{
	        obj=super.clone();
	    }catch(CloneNotSupportedException ex){
	        System.out.println(" no es pot duplicar");
	    }
	    return obj;
	}	
	/**
	 * Seteja l'identificador del registre a 0.
	 * S'usa per inserir nous registres a la base de dades
	 */
	public void idNouRegistre(){
		idHost=0;
	}		
}