package gwpkg;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/** Implementa les operacions de la lgica de negoci pertinents a l'entitat Host
 */
@Stateful
public class HostServeis implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Entity manager JPA usat per les operacions de persistncia de l'entitat host
	 */
	@PersistenceContext(unitName="gwpkg")
	private EntityManager em;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public HostServeis(){
	}
	
	/**
	 * Crea una instncia de Host Buida.
	 * Es necessita per donar d'alta nous registres a la base de dades
	 * @post instncia de Host buida
	 * @return instncia de Host buida
	 */
	public Host hostBuit(){
		Host h = new Host();
		return h;
	}
	/**
	 * Clona el registre actual
	 */
	public Host clonarHost(Host hostOriginal){
		Host hostClonat = hostBuit();
		hostClonat.setIdHost(hostOriginal.getIdHost());
		hostClonat.setInhabilitat(hostOriginal.getInhabilitat());
		hostClonat.setNomHost(hostOriginal.getNomHost());
		hostClonat.setDescripcioHost(hostOriginal.getDescripcioHost());
		List<Perfil> l = new ArrayList<Perfil>();
		hostClonat.setPerfils(l);
		hostClonat.getPerfils().addAll(hostOriginal.getPerfils());
		return hostClonat;
	}
	/**
	 * Retorna la llista de Hosts existents a la base de dades.
	 * Filtra o no els resultats per aquells que estan inhabilitats.
	 * @pre	Existeix mostrarInhabilitats
	 * @post  List<Host> Llista Hosts filtrada
	 * @return  List<Host>
	 */	
	public List<Host> llista(Boolean inhabilitats){
		String strQr = "SELECT h FROM Host h";		
		if (!inhabilitats){
			strQr = strQr + " WHERE h.inhabilitat = :MOSTRAR_INHABILITATS";
		}
		Query qr = em.createQuery(strQr);			
		if (!inhabilitats){
			qr.setParameter("MOSTRAR_INHABILITATS",inhabilitats);			
		}
		@SuppressWarnings("unchecked")
		List<Host> llistaInicial = qr.getResultList();
		
		return llistaInicial;
	}
	/**
	 * Persisteix una instncia de l'entitat Host
	 * @pre	Existeix hostActiu
	 * @post  hostActiu persistit
	 */	
	public void gravarHost(Host hostActiu){
			em.merge(hostActiu);
	}
	/**
	 * Elimina una instncia de l'entitat host
	 * @pre	Existeix hostActiu
	 * @post  hostActiu eliminat
	 */	
	public void esborrarHost(Host hostActiu){
		em.remove(em.merge(hostActiu));
		}

}
