package gwpkg;

import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.ejb.EJB;
import java.io.Serializable;
import java.util.List;
import javax.annotation.PostConstruct;

/**
 * Bean de suport (managed bean) per hostPage.xhtml.
 * Cont els atributs i mtodes que implementen el comportament de
 * la interfcie d'usuari.
 */
@SessionScoped
@Named("hostView")
public class HostView implements Serializable{
	private static final long serialVersionUID = 1L;

	/**
	 * Implementa la lgica de negoci referent a l'entitat Host.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private HostServeis hostServeis;
	/**
	 * Instncia de l'entitat Host amb la que s'est treballant.
	 */
	@EJB
	private Host hostActiu;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Perfil.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private PerfilServeis perfilServeis;
	/**
	 * Instncia de l'entitat Host amb la que s'inicialitza un nou registre.
	 */
	private Host hostBuit;
	/**
	 * Instncia de l'entitat Host que cont l'ltim estat persistit.
	 * Es fa servir per cancellar les ltimes accions fetes i recuperar l'ltim
	 * estat persistit.
	 */
	private Host hostBackUp;
	
	/**
	 * Apuntador a l'element de la llista de Hosts que cont
	 * la base de dades 
	 */
	private Integer indexLlistaHosts;
	/**
	 * Apuntador a l'element actiu de la llista de Perfils assignats
	 */
	private Integer indexLlistaPerfilsAssignats;
	/**
	 * Apuntador a l'element actiu de la llista de Perfils disponibles
	 */
	private Integer indexLlistaPerfilsDisponibles;
	
	/**
	 * Llista contenint els Hosts disponibles a la base de dades.
	 */
	private List<Host> llistaHosts;
	/**
	 * Llista contenint els perfils disponibles a la base de dades.
	 */
	private List<Perfil> perfilsDisponibles;
	
	/**
	 * true: A la llista de Hosts disponibles s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 * false: A la llista de Hosts disponibles NO s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 */
	private Boolean mostrarInhabilitats=false;
	/**
	 * true: Al inicialitzar un registre nou es clona el registre actiu
	 */
	private Boolean clonarHost=false;
	/** Flag per renderitzar o no la <div> d'errors a la pgina
	 * false: no es mmostra la <div> d'errors. No s'ha capturat cap excepci.
	 */
	private Boolean displayError=false;
	/**
	 * Mtode que ha capturat la excepci.
	 */
	private String metodeError="";
	/**
	 * Causa de la excepci.(e.getCause().toString();)
	 */
	private String causaError="";
	
	
	
	/**Constructor buit exigit per l'especificaci JSF2
	 */
	public HostView(){
	}
	
	// -------- HOST GETERS I SETTERS
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getDisplayError(){
		return displayError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDisplayError(Boolean displayError){
		this.displayError=displayError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getMetodeError(){
		return metodeError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMetodeError(String metodeError){
		this.metodeError=metodeError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getCausaError(){
		return causaError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setCausaError(String causaError){
		this.causaError=causaError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Integer getIndexLlistaHosts(){
		return indexLlistaHosts;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaHosts(Integer indexLlistaHosts){
		this.indexLlistaHosts=indexLlistaHosts;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsAssignats(Integer indexLlistaPerfilsAssignats){
		this.indexLlistaPerfilsAssignats=indexLlistaPerfilsAssignats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsDisponibles(Integer indexLlistaPerfilsDisponibles){
		this.indexLlistaPerfilsDisponibles=indexLlistaPerfilsDisponibles;		
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Host getHostActiu(){
		return hostActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setHostActiu(Host hostActiu){
		this.hostActiu=hostActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setHostBuit(Host hostBuit){
		this.hostBuit=hostBuit;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Host getHostBuit(){
		return hostBuit;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setHostBackUp(Host hostBackUp){
		this.hostBackUp=hostBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Host getHostBackUp(){
		return hostBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Host> getLlistaHosts(){
		return llistaHosts;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setLlistaHosts(List<Host> llistaHosts){
		this.llistaHosts=llistaHosts;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMostrarInhabilitats(Boolean mostrarInhabilitats){
		this.mostrarInhabilitats = mostrarInhabilitats;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getMostrarInhabilitats(){
		return this.mostrarInhabilitats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setClonarHost(Boolean clonarHost){
		this.clonarHost = clonarHost;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getClonarHost(){
		return this.clonarHost;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilsDisponibles(){
		perfilsDisponibles = perfilServeis.llista(false);	
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Perfil> getPerfilsDisponibles(){
		return perfilsDisponibles;
	}
	
	//--------------- HOST ACCIONS
	
	/**
	 * Respn a la petici de l'usuari per Editar el Host escollit
	 * @pre: existeix llistaHosts
	 * 		 existeix indexLlistaHosts
	 * @post: hostActiu=llistaHosts(indexLlistaHosts) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarHost(){
		try {
			canviaHostActiu(getIndexLlistaHosts());		
		} catch	(Exception e) {
			error("editarHost",e);
		}	
		return "host";
	}
	
	/**
	 * Respn a la petici de l'usuari per Editar el Host escollit
	 * @pre: existeix llistaHosts
	 * 		 existeix indexLlistaHosts
	 * @post: hostActiu=llistaPerfils(indexLlistaHosts) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public void canviaHostActiu(Integer indexLlistaHosts){
		if (indexLlistaHosts >= 0) {
			setIndexLlistaHosts(indexLlistaHosts);
			setHostActiu(getLlistaHosts().get(indexLlistaHosts));
		} else {
			setIndexLlistaHosts(0);
			setHostActiu(getHostBuit());
		}
		setHostBackUp(hostServeis.clonarHost(hostActiu));
	}

	/**
	 * Respn a la petici de l'usuari per Persistir el Host actiu
	 * @pre: existeix hostActiu
	 * @post: s'ha persistit  el host actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarHost(){
		try {
			hostServeis.gravarHost(getHostActiu());
			if (getHostActiu().getIdHost()==0) {
				ompleLlistaHosts();
			}
		} catch (Exception e) {
			error("gravarHost",e);
		}
		return "host";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar el host actiu
	 * @pre: existeix hostActiu
	 * @post: s'ha eliminat el host actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String esborrarHost(){
		try {
			hostServeis.esborrarHost(getHostActiu());
			getLlistaHosts().remove(getHostActiu());
			setIndexLlistaHosts(getIndexLlistaHosts()-1);
			canviaHostActiu(getIndexLlistaHosts());
		} catch (Exception e) {
			error("esborrarHost",e);
		}
		return "host";			
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Host
	 * @pre: atribut clonarHost
	 * @post: si clonarHost = true
	 * 				clona hostActiu amb id = 0 (nou registre)
	 * 		  si clonarHost = false
	 * 				hostActiu = hostBuit	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String nouHost(){
		try {
			if (!clonarHost) {
				setHostActiu(hostServeis.hostBuit());
			} else {
				Host nouh = hostServeis.clonarHost(hostActiu);
				setHostActiu(nouh);
				getHostActiu().idNouRegistre();
			}		
			setHostBackUp(hostServeis.clonarHost(hostActiu));
		} catch (Exception e) {
			error("nouHost",e);
		}	
		return "host";
	}
	
	/**
	 * Respn a la petici de l'usuari per Cancelar les modificacions fetes a Host
	 * desde la ltima vegada que ha estat persistida.
	 * @pre: existeix hostBackup
	 *       existeix hostActiu
	 * @post: hostActiu=hostBackup
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String cancelarHost(){
		try {
			setHostActiu( hostServeis.clonarHost(hostBackUp));
			getLlistaHosts().set(getIndexLlistaHosts(), getHostActiu());
		} catch	(Exception e) {
			error("cancelarHost",e);
		}	
		return "host";
	}
	
	/**
	 * Respn a la petici de l'usuari per Filtrar la llista de Hosts disponibles
	 * segons els criteris especificats.
	 * @pre: existeix llistaHosts
	 *       existeix mostrarInhabilitats
	 * @post: llistaHosts filtrada
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String filtraLlistaHosts(){
		try {
			ompleLlistaHosts();
		} catch	(Exception e) {
			error("filtraLlistaHosts",e);
		}	
		return "host";	
	}
	/**
	 * Respn a la petici de l'usuari per deassignar un perfil al host actiu
	 * @pre: existeix hostActiu
	 *       existeix indexLlistaPerfilsAssignats
	 * @post: paquet  indexLlistaPerfilsAssignats desassignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String desassignarPerfil(){
		try {
			List<Perfil> llistaPerfils = hostActiu.getPerfils();
			Perfil perfil = llistaPerfils.get(indexLlistaPerfilsAssignats);
			llistaPerfils.remove(perfil);
		} catch	(Exception e) {
			error("desassignarPerfil",e);
		}	
		return "host";
	}
		/**
		 * Respn a la petici de l'usuari per assignar un perfil al Host actiu
		 * @pre: existeix hostActiu
		 *       existeix indexLlistaPerfilsDisponibles
		 * @post: paquet  indexLlistaPerfilsDisponibles assignat
		 * @return String pgina a displayar desprs de l'execuci del mtode.
		 */
	 public String assignarPerfil(){
		try {
			List<Perfil> llistaPerfils = hostActiu.getPerfils();
			Perfil perfil = perfilsDisponibles.get(indexLlistaPerfilsDisponibles);
			if (!(llistaPerfils.contains(perfil))) {
				llistaPerfils.add(perfil);
			}
		} catch	(Exception e) {
			error("assignarPerfil",e);
		}	
		return "host";
	}
		/**
		 * Displaya l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
		 */
		private void error(String metode,Exception e){
			setDisplayError(true);
			metodeError=metode;

			if (e.getCause()==null){
				causaError=e.getStackTrace()[0].toString();
			} else {
				causaError=e.getCause().toString();
			}
		}

		/**
		 * Esborra el display de l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
		 */
		public void noDisplayError(){
			setDisplayError(false);
		}
	 
	
	//------------- POSTCONSTRUCT
	
		/**
		 * S'executa per inicialitzar les estructures de la pgina
		 */	
	@PostConstruct
	private void ompleLlistaHosts(){
		setLlistaHosts(hostServeis.llista(getMostrarInhabilitats()));
		setHostBuit(hostServeis.hostBuit());
		setIndexLlistaHosts(0);
		if (getLlistaHosts().size()>0) {
			setHostActiu(getLlistaHosts().get(getIndexLlistaHosts()));
		} else {
			setHostActiu(getHostBuit());
		}
		setHostBackUp(hostServeis.clonarHost(hostActiu));
		setPerfilsDisponibles();
	}
}
