package gwpkg;

import java.io.Serializable;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.ManyToMany;
import javax.ejb.Stateful;
import javax.persistence.*;
import java.util.List;
import java.util.ArrayList;

/**
* Classe que representa el model de dades de l'entitat PAQUET <P>
* Una instncia d'aquesta entitat es correspon a un paquet installable de programari
* Inclou tots els comanaments de comprovaci, installaci, desinstallaci i actualitzaci
* necessaris per la gesti del paquet.
*/
@Entity
@Stateful
@Table(name="paquet")
public class Paquet implements Serializable{
	private static final long serialVersionUID = 1L;

/** Codi identificador del paquet. s autonumric. */
@Id
@GeneratedValue(strategy=GenerationType.IDENTITY)
private Integer idPaquet=0;
/** Nom del paquet. Descriptor mnemotcnic d'entre 0 i 15 caracters.*/ 
private String nomPaquet;
/** Descripci del paquet. Descripci general de les especifitats d'un paquet.
 * Mxim 100 caracters */
private String descripcioPaquet;
/** Estat del paquet. Habilitat/Deshabilitat.
 * Si est habilitat es mostrar a els grids de selecci de paquets.
 * Si est deshabilitat no es mostrar als grids de selecci.
 * L'estat deshabilitat no comporta cap altre comportament.
 * BOOLEAN */ 
private Boolean inhabilitat=false;
/**
 * Integer generat per l'usuari (entre 0 i 100) que indica la revisi del paquet
 * S'ha d'augmentar quan es genera una revisi del paquet
 * BOOLEAN */ 
private Integer revisio=0;
/**Si s TRUE la installaci del paquet anir seguida d'un reinici.
 * BOOLEAN */ 
private Boolean reiniciar=false;
/**
 *Integer generat per l'usuari (entre 0 i 10) que indica la versi del paquet
 *Com ms gran el valor abans s'installar el paquet
 */
private Integer prioritat=0;

/**
 * bi-directional many-to-one association to Comprovacio
 */
@OneToMany(mappedBy="paquet", fetch=FetchType.EAGER)
private List<Comprovacio> comprovacionsPaquet = new ArrayList<Comprovacio>();

/**
 * bi-directional many-to-one association to Instalacio
 */
@OneToMany(mappedBy="paquet", fetch=FetchType.EAGER)
private List<Instalacio> instalacionsPaquet = new ArrayList<Instalacio>();

/**
 * bi-directional many-to-one association to Actualitzacio
 */
@OneToMany(mappedBy="paquet", fetch=FetchType.EAGER)
private List<Actualitzacio> actualitzacionsPaquet = new ArrayList<Actualitzacio>();

/**
 * bi-directional many-to-one association to Desinstalacio
 */
@OneToMany(mappedBy="paquet", fetch=FetchType.EAGER)
private List<Desinstalacio> desinstalacionsPaquet = new ArrayList<Desinstalacio>();

/**
 * bi-directional many-to-many association to Perfil
 */
@ManyToMany(mappedBy="paquets", fetch=FetchType.EAGER)
private List<Perfil> perfils;

/**Constructor buit exigit per l'especificaci JPA2/EJB3
 */
public Paquet(){
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Integer getIdPaquet(){
	return idPaquet;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getNomPaquet(){
	return nomPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setNomPaquet(String nomPaquet){
	this.nomPaquet=nomPaquet;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public String getDescripcioPaquet(){
	return descripcioPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setDescripcioPaquet(String descripcioPaquet){
	this.descripcioPaquet=descripcioPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setInhabilitat(Boolean inhabilitat){
	this.inhabilitat=inhabilitat;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getInhabilitat(){
	return this.inhabilitat;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Integer getRevisio(){
	return revisio;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setRevisio(Integer revisio){
	this.revisio=revisio;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setReiniciar(Boolean reiniciar){
	this.reiniciar=reiniciar;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Boolean getReiniciar(){
	return this.reiniciar;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public Integer getPrioritat(){
	return prioritat;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setPrioritat(Integer prioritat){
	this.prioritat=prioritat;
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public List<Comprovacio> getComprovacionsPaquet(){
	return comprovacionsPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setComprovacionsPaquet(List<Comprovacio> comprovacionsPaquet){
	this.comprovacionsPaquet=comprovacionsPaquet;
}

/**Geter exigit per l'especificaci JPA2/EJB3
 */
public List<Instalacio> getInstalacionsPaquet(){
	return instalacionsPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setInstalacionsPaquet(List<Instalacio> instalacionsPaquet){
	this.instalacionsPaquet=instalacionsPaquet;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public List<Actualitzacio> getActualitzacionsPaquet(){
	return actualitzacionsPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setActualitzacionsPaquet(List<Actualitzacio> actualitzacionsPaquet){
	this.actualitzacionsPaquet=actualitzacionsPaquet;
}
/**Geter exigit per l'especificaci JPA2/EJB3
 */
public List<Desinstalacio> getDesinstalacionsPaquet(){
	return desinstalacionsPaquet;
}
/**Seter exigit per l'especificaci JPA2/EJB3
 */
public void setDesinstalacionsPaquet(List<Desinstalacio> desinstalacionsPaquet){
	this.desinstalacionsPaquet=desinstalacionsPaquet;
}

/**
 * Seteja l'identificador de l registre a 0.
 * S'usa per inserir nous registres a la base de dades
 */
public void idNouRegistre(){
	idPaquet=0;
}

/**
 * Clona el registre actual
 */
public Object clone(){
    Object obj=null;
    try{
        obj=super.clone();
    }catch(CloneNotSupportedException ex){
        System.out.println(" no es pot duplicar");
    }
    return obj;
}


}
