package gwpkg;

import java.io.Serializable;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/** Implementa les operacions de la lgica de negoci pertinents a l'entitat paquet
 */
@Stateful
public class PaquetServeis implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Entity manager JPA usat per les operacions de persistncia de l'entitat configuraci
	 */
	@PersistenceContext(unitName="gwpkg")
	private EntityManager em;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public PaquetServeis(){
	}
	/**
	 * Crea una instncia de paquet Buida.
	 * Es necessita per donar d'alta nous registres a la base de dades
	 * @post instncia de paquet buida
	 * @return instncia de paquet buida
	 */
	public Paquet paquetBuit(){
		Paquet pb = new Paquet();
		return pb;
	}
	/**
	 * Retorna la llista de paquets existents a la base de dades.
	 * Filtra o no els resultats per aquells que estan inhabilitats.
	 * @pre	Existeix mostrarInhabilitats
	 * @post  List<Paquet> Llista paquets filtrada
	 * @return  List<Paquet>
	 */	
	public List<Paquet> llista(Boolean inhabilitats){
		String strQr = "SELECT p FROM Paquet p";		
		if (!inhabilitats){
			strQr = strQr + " WHERE p.inhabilitat = :MOSTRAR_INHABILITATS";
		}
		Query qr = em.createQuery(strQr);			
		if (!inhabilitats){
			qr.setParameter("MOSTRAR_INHABILITATS",inhabilitats);			
		}
		@SuppressWarnings("unchecked")
		List<Paquet> llistaInicial = qr.getResultList();
		
		return llistaInicial;
	}
	/**
	 * Persisteix una instncia de l'entitat paquet
	 * @pre	Existeix paquetActiu
	 * @post  paquetActiu persistit
	 */	
	public void gravarPaquet(Paquet paquetActiu){
		em.merge(paquetActiu);
	}
	/**
	 * Elimina una instncia de l'entitat paquet
	 * @pre	Existeix paquetActiu
	 * @post  paquetActiu eliminat
	 */	
	public void esborrarPaquet(Paquet paquetActiu){
		em.remove(em.merge(paquetActiu));
		}

	/**
	 * Obt la llista de Comprovacions assignades al paquet actiu
	 * @pre	Existeix paquetActiu
	 * @post  Retorna la llista de Comprovacions assignades al paquet
	 */	
	public List<Comprovacio> llistaComprovacions(Paquet paquetActiu) {
		String strQr = "SELECT c FROM Comprovacio c WHERE c.paquet = :id";
		Query qr = em.createQuery(strQr);
		qr.setParameter("id", paquetActiu);
		@SuppressWarnings("unchecked")		
		List<Comprovacio> comprovacionsPaquet = qr.getResultList();
		return comprovacionsPaquet;
	}
	/**
	 * Obt la llista de Instalacions assignades al paquet actiu
	 * @pre	Existeix paquetActiu
	 * @post  Retorna la llista de Instalacions assignades al paquet
	 */	
	public List<Instalacio> llistaInstalacions(Paquet paquetActiu) {
		String strQr = "SELECT c FROM Instalacio c WHERE c.paquet = :id";
		Query qr = em.createQuery(strQr);
		qr.setParameter("id", paquetActiu);
		@SuppressWarnings("unchecked")		
		List<Instalacio> instalacionsPaquet = qr.getResultList();
		return instalacionsPaquet;
	}
	/**
	 * Obt la llista de Actualitzacions assignades al paquet actiu
	 * @pre	Existeix paquetActiu
	 * @post  Retorna la llista de Actualitzacions assignades al paquet
	 */	
	public List<Actualitzacio> llistaActualitzacions(Paquet paquetActiu) {
		String strQr = "SELECT c FROM Actualitzacio c WHERE c.paquet = :id";
		Query qr = em.createQuery(strQr);
		qr.setParameter("id", paquetActiu);
		@SuppressWarnings("unchecked")		
		List<Actualitzacio> actualitzacionsPaquet = qr.getResultList();
		return actualitzacionsPaquet;
	}
	/**
	 * Obt la llista de Desactualitzacions assignades al paquet actiu
	 * @pre	Existeix paquetActiu
	 * @post  Retorna la llista de Desactualitzacions assignades al paquet
	 */	
	public List<Desinstalacio> llistaDesinstalacions(Paquet paquetActiu) {
		String strQr = "SELECT c FROM Desinstalacio c WHERE c.paquet = :id";
		Query qr = em.createQuery(strQr);
		qr.setParameter("id", paquetActiu);
		@SuppressWarnings("unchecked")		
		List<Desinstalacio> desinstalacionsPaquet = qr.getResultList();
		return desinstalacionsPaquet;
	}
}
