package gwpkg;

import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.ejb.EJB;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.PostConstruct;

/**
 * Bean de suport (managed bean) per paquetPage.xhtml.
 * Cont els atributs i mtodes que implementen el comportament de
 * la interfcie d'usuari.
 */
@SessionScoped
@Named("paquetView")
public class PaquetView implements Serializable{
	private static final long serialVersionUID = 1L;
	
	//---------- PAQUET	

	/**
	 * Instncia de l'entitat Paquet amb la que s'est treballant.
	 */
	@EJB
	private Paquet paquetActiu;
	/**
	 * Instncia de l'entitat Paquet amb la que s'inicialitza un nou registre.
	 */
	private Paquet paquetBuit;
	/**
	 * Instncia de l'entitat Paquet que cont l'ltim estat persistit.
	 * Es fa servir per cancellar les ltimes accions fetes i recuperar l'ltim
	 * estat persistit.
	 */
	private Paquet paquetBackUp;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Paquet.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private PaquetServeis paquetServeis;
	/**
	 * Llista contenint les Configuracio disponibles a la base de dades.
	 */
	private List<Paquet> llistaPaquets;
	/**
	 * Apuntador a l'element de la llista de Paquets disponibles que cont
	 * la configuraci amb la que s'est treballant 
	 */
	private Integer indexLlistaPaquets=0;

	/**
	 * true: A la llista de Paquets disponibles s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 * false: A la llista de Paquets disponibles NO s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 */
	private Boolean mostrarInhabilitats=false;
	/**
	 * true: Al inicialitzar un registre nou es clona el registre actiu
	 */
	private Boolean clonarPaquet=false;

	//---------- COMPROVACIO
	@EJB
	private ComprovacioServeis comprovacioServeis; 
	@EJB
	private Comprovacio comprovacioActiva;
	private Comprovacio comprovacioActivaBackUp;
	private Comprovacio comprovacioBuida;
	private Integer indexLlistaComprovacions=0;
	private Boolean clonarComprovacio=false;
	
	//---------- INSTALACIO
	@EJB	
	private InstalacioServeis instalacioServeis; 
	@EJB
	private Instalacio instalacioActiva;
	private Instalacio instalacioActivaBackUp;
	private Instalacio instalacioBuida;	
	private Integer indexLlistaInstalacions=0;
	private Boolean clonarInstalacio=false;
	
	//---------- ACTUALITZACIO
	@EJB
	private ActualitzacioServeis actualitzacioServeis; 
	@EJB
	private Actualitzacio actualitzacioActiva;
	private Actualitzacio actualitzacioActivaBackUp;
	private Actualitzacio actualitzacioBuida;
	private Integer indexLlistaActualitzacions=0;
	private Boolean clonarActualitzacio=false;
	
	//---------- DESINSTALACIO
	@EJB
	private DesinstalacioServeis desinstalacioServeis; 
	@EJB
	private Desinstalacio desinstalacioActiva;
	private Desinstalacio desinstalacioActivaBackUp;
	private Desinstalacio desinstalacioBuida;
	private Integer indexLlistaDesinstalacions=0;
	private Boolean clonarDesinstalacio=false;

	/** Flag per renderitzar o no la <div> d'errors a la pgina
	 * false: no es mmostra la <div> d'errors. No s'ha capturat cap excepci.
	 */
	private Boolean displayError=false;
	/**
	 * Mtode que ha capturat la excepci.
	 */
	private String metodeError="";
	/**
	 * Causa de la excepci.(e.getCause().toString();)
	 */
	private String causaError="";
	
	
	
	/**Constructor buit exigit per l'especificaci JSF2
	 */
	public PaquetView(){
	}
	
	// -------- PAQUETS GETERS I SETTERS

	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getDisplayError(){
		return displayError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDisplayError(Boolean displayError){
		this.displayError=displayError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getMetodeError(){
		return metodeError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMetodeError(String metodeError){
		this.metodeError=metodeError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getCausaError(){
		return causaError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setCausaError(String causaError){
		this.causaError=causaError;
	}
	
	/**Geter exigit per l'especificaci JSF2
	 */
	public Integer getIndexLlistaPaquets(){
		return indexLlistaPaquets;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPaquets(Integer indexLlistaPaquets){
		this.indexLlistaPaquets=indexLlistaPaquets;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Paquet getPaquetActiu(){
		return paquetActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPaquetActiu(Paquet paquetActiu){
		this.paquetActiu=paquetActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPaquetBuit(Paquet paquetBuit){
		this.paquetBuit=paquetBuit;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Paquet getPaquetBuit(){
		return paquetBuit;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPaquetBackUp(Paquet paquetBackUp){
		this.paquetBackUp=paquetBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Paquet getPaquetBackUp(){
		return paquetBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Paquet> getLlistaPaquets(){
		return llistaPaquets;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setLlistaPaquets(List<Paquet> llistaPaquets){
		this.llistaPaquets=llistaPaquets;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMostrarInhabilitats(Boolean mostrarInhabilitats){
		this.mostrarInhabilitats = mostrarInhabilitats;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getMostrarInhabilitats(){
		return this.mostrarInhabilitats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setClonarPaquet(Boolean clonarPaquet){
		this.clonarPaquet = clonarPaquet;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getClonarPaquet(){
		return this.clonarPaquet;
	}

	//--------------- COMPROVACIONS GETERS I SETTERS
	public Comprovacio getComprovacioActiva(){
		return comprovacioActiva;
	}
	public void setComprovacioActiva(Comprovacio comprovacioActiva){
		this.comprovacioActiva=comprovacioActiva;
	}
	public Integer getIndexLlistaComprovacions(){
		return indexLlistaComprovacions;
	}
	public void setIndexLlistaComprovacions(Integer indexLlistaComprovacions){
		this.indexLlistaComprovacions=indexLlistaComprovacions;
	}
	public Comprovacio getComprovacioActivaBackUp(){
		return this.comprovacioActivaBackUp;
	}
	public void setComprovacioActivaBackUp(Comprovacio comprovacioActivaBackUp){
		this.comprovacioActivaBackUp=comprovacioActivaBackUp;
	}
	public Comprovacio getComprovacioBuida(){
		return this.comprovacioBuida;
	}
	public void setComprovacioBuida(Comprovacio comprovacioBuida){
		this.comprovacioBuida=comprovacioBuida;
	}
	public void setClonarComprovacio(Boolean clonarComprovacio){
		this.clonarComprovacio = clonarComprovacio;
	}
	public Boolean getClonarComprovacio(){
		return this.clonarComprovacio;
	}
	//--------------- INSTALACIONS GETERS I SETTERS
	public Instalacio getInstalacioActiva(){
		return instalacioActiva;
	}
	public void setInstalacioActiva(Instalacio instalacioActiva){
		this.instalacioActiva=instalacioActiva;
	}	
	public Integer getIndexLlistaInstalacions(){
		return indexLlistaInstalacions;
	}
	public void setIndexLlistaInstalacions(Integer indexLlistaInstalacions){
		this.indexLlistaInstalacions=indexLlistaInstalacions;
	}
	public Instalacio getInstalacioActivaBackUp(){
		return this.instalacioActivaBackUp;
	}
	public void setInstalacioActivaBackUp(Instalacio instalacioActivaBackUp){
		this.instalacioActivaBackUp=instalacioActivaBackUp;
	}
	public Instalacio getInstalacioBuida(){
		return this.instalacioBuida;
	}
	public void setInstalacioBuida(Instalacio instalacioBuida){
		this.instalacioBuida=instalacioBuida;
	}
	public void setClonarInstalacio(Boolean clonarInstalacio){
		this.clonarInstalacio = clonarInstalacio;
	}
	public Boolean getClonarInstalacio(){
		return this.clonarInstalacio;
	}

	//--------------- ACTUALITZACIO GETERS I SETTERS
	public Actualitzacio getActualitzacioActiva(){
		return actualitzacioActiva;
	}
	public void setActualitzacioActiva(Actualitzacio actualitzacioActiva){
		this.actualitzacioActiva=actualitzacioActiva;
	}	
	public Integer getIndexLlistaActualitzacions(){
		return indexLlistaActualitzacions;
	}
	public void setIndexLlistaActualitzacions(Integer indexLlistaActualitzacions){
		this.indexLlistaActualitzacions=indexLlistaActualitzacions;
	}
	public Actualitzacio getActualitzacioActivaBackUp(){
		return this.actualitzacioActivaBackUp;
	}
	public void setActualitzacioActivaBackUp(Actualitzacio actualitzacioActivaBackUp){
		this.actualitzacioActivaBackUp=actualitzacioActivaBackUp;
	}
	public Actualitzacio getActualitzacioBuida(){
		return this.actualitzacioBuida;
	}
	public void setActualitzacioBuida(Actualitzacio actualitzacioBuida){
		this.actualitzacioBuida=actualitzacioBuida;
	}
	public void setClonarActualitzacio(Boolean clonarActualitzacio){
		this.clonarActualitzacio = clonarActualitzacio;
	}
	public Boolean getClonarActualitzacio(){
		return this.clonarActualitzacio;
	}

	//--------------- DESINSTALACIO GETERS I SETTERS
	public Desinstalacio getDesinstalacioActiva(){
		return desinstalacioActiva;
	}
	public void setDesinstalacioActiva(Desinstalacio desinstalacioActiva){
		this.desinstalacioActiva=desinstalacioActiva;
	}	
	public Integer getIndexLlistaDesinstalacions(){
		return indexLlistaDesinstalacions;
	}
	public void setIndexLlistaDesinstalacions(Integer indexLlistaDesinstalacions){
		this.indexLlistaDesinstalacions=indexLlistaDesinstalacions;
	}
	public Desinstalacio getDesinstalacioActivaBackUp(){
		return this.desinstalacioActivaBackUp;
	}
	public void setDesinstalacioActivaBackUp(Desinstalacio desinstalacioActivaBackUp){
		this.desinstalacioActivaBackUp=desinstalacioActivaBackUp;
	}
	public Desinstalacio getDesinstalacioBuida(){
		return this.desinstalacioBuida;
	}
	public void setDesinstalacioBuida(Desinstalacio desinstalacioBuida){
		this.desinstalacioBuida=desinstalacioBuida;
	}
	public void setClonarDesinstalacio(Boolean clonarDesinstalacio){
		this.clonarDesinstalacio = clonarDesinstalacio;
	}
	public Boolean getClonarDesinstalacio(){
		return this.clonarDesinstalacio;
	}
	
	//------------------------------------------------------------
	// MTODES PER IMPLEMENTAR EL COMPORTAMENT DE LA INTERFCIE D'USUARI
	//------------------------------------------------------------

	//--------------- PAQUETS
	
	/**
	 * Respn a la petici de l'usuari per Editar el paquet escollit
	 * @pre: existeix llistaPaquets
	 * 		 existeix indexLlistaPaquets
	 * @post: paquetActiu=llistaPaquets(indexLlistaPaquets) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarPaquet(){
		try {
			canviaPaquetActiu(getIndexLlistaPaquets());		
		} catch	(Exception e) {
			error("editarPaquet",e);
		}	
		return "paquet";
	}
	
	/**
	 * Respn a la petici de l'usuari per Editar el paquet escollit
	 * @pre: existeix llistaPaquets
	 * 		 existeix indexLlistaPaquets
	 * @post: paquetActiu=llistaPaquets(indexLlistaPaquets) 
	 */
	public void canviaPaquetActiu(Integer indexLlistaPaquets){
		if (indexLlistaPaquets >= 0) {
			setIndexLlistaPaquets(indexLlistaPaquets);
			setPaquetActiu(getLlistaPaquets().get(indexLlistaPaquets));
		} else {
			setIndexLlistaPaquets(0);
			setPaquetActiu(getPaquetBuit());
		}
		setPaquetBackUp((Paquet) getPaquetActiu().clone());
	}

	/**
	 * Respn a la petici de l'usuari per Persistir el paquet actiu
	 * @pre: existeix paquetActiu
	 * @post: s'ha persistit  el paquet actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarPaquet(){	
		try {
			paquetServeis.gravarPaquet(getPaquetActiu());
			ompleLlistaPaquets();
		} catch (Exception e) {
			error("gravarPaquet",e);
		}
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar el paquet actiu
	 * @pre: existeix paquetActiu
	 * @post: s'ha eliminat el paquet actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String esborrarPaquet(){
		try {
			paquetServeis.esborrarPaquet(getPaquetActiu());
			getLlistaPaquets().remove(getPaquetActiu());
			setIndexLlistaPaquets(getIndexLlistaPaquets()-1);
			canviaPaquetActiu(getIndexLlistaPaquets());
		} catch (Exception e) {
			error("esborrarPaquet",e);
		}
		canviarComprovacioActiva();
		canviarActualitzacioActiva();
		canviarInstalacioActiva();
		canviarDesinstalacioActiva();
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Paquet
	 * @pre: atribut clonarPaquet
	 * @post: si clonarPaquet = true
	 * 				clona paquetActiu amb id = 0 (nou registre)
	 * 		  si clonarPaquet = false
	 * 				paquetActiu = paquetBuit	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String nouPaquet(){
		try {
			if (!clonarPaquet) {
				setPaquetActiu(getPaquetBuit());
			} else {
				Paquet noup = (Paquet) getPaquetActiu().clone();
				setPaquetActiu(noup);
				getPaquetActiu().idNouRegistre();
			}
			setPaquetBackUp( (Paquet) this.paquetActiu.clone());
		} catch (Exception e) {
			error("nouPaquet",e);
		}
		canviarComprovacioActiva();
		canviarActualitzacioActiva();
		canviarInstalacioActiva();
		canviarDesinstalacioActiva();		
		return "paquet";
	}
	
	/**
	 * Respn a la petici de l'usuari per Cancelar les modificacions fetes a Paquet
	 * desde la ltima vegada que ha estat persistida.
	 * @pre: existeix paquetBackup
	 *       existeix paquetActiu
	 * @post: paquetActiu=paquetBackup
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String cancelarPaquet(){
		try {
			setPaquetActiu( (Paquet) this.paquetBackUp.clone());
		} catch	(Exception e) {
			error("cancelarPaquet",e);
		}	
		return "paquet";
	}
	
	/**
	 * Respn a la petici de l'usuari per Filtrar la llista de Paquets disponibles
	 * segons els criteris especificats.
	 * @pre: existeix llistaPaquets
	 *       existeix mostrarInhabilitats
	 * @post: llistaPaquets filtrada
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String filtraLlistaPaquets(){
		try {
			ompleLlistaPaquets();
		} catch	(Exception e) {
			error("filtraLlistaPaquets",e);
		}	
		return "paquet";	
	}
	
	//--------------- COMPROVACIONS ACCIONS
	/**
	 * Actuliatza el registre de comprovaci actual per la seva eventual edici
	 * @pre: existeix paquetActiu
	 *       existeix indexLlistaComprovacions
	 * @post: comprovacioActiva actualitzada
	 */	
	public void canviarComprovacioActiva(){
		if (getPaquetActiu().getComprovacionsPaquet().size()>0){
			setComprovacioActiva(getPaquetActiu().getComprovacionsPaquet().get(indexLlistaComprovacions));
		} else {
			setComprovacioActiva(getComprovacioBuida());
			getComprovacioActiva().setPaquet(getPaquetActiu());
		}
	}

	/**
	 * Respn a la petici de l'usuari per Editar el registre de Comprovaci escollit
	 * @pre: existeix comprovacioActiva
	 * @post: Comprovaci escollida carregada a la interfcie d'usuari
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarComprovacio(){
		canviarComprovacioActiva();
		return "paquet";			
	}

	/**
	 * Respn a la petici de l'usuari per Persistir la Comprovacio activa
	 * @pre: existeix comprovacioActiva
	 * @post: s'ha persistit  la Comprovacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarComprovacio(){
		try {
			if (paquetActiu.getComprovacionsPaquet() != null) {
				List<Comprovacio> llistaComprovacions = paquetActiu.getComprovacionsPaquet();
				if (!(llistaComprovacions.contains(comprovacioActiva))) {
					llistaComprovacions.add(comprovacioActiva);
				} else {
					comprovacioServeis.gravarComprovacio(comprovacioActiva);
				}
			} else {
				List<Comprovacio> llistaComprovacions = new ArrayList<Comprovacio>();	
				paquetActiu.setComprovacionsPaquet(llistaComprovacions);			
				llistaComprovacions.add(comprovacioActiva);
			}

		} catch	(Exception e) {
			error("gravarComprovacio",e);
		}	
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la Comprovacio activa
	 * @pre: existeix comprovacioActiva
	 * @post: s'ha eliminat la Comprovacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String esborrarComprovacio(){		
		comprovacioServeis.esborrarComprovacio(getComprovacioActiva());
		getPaquetActiu().getComprovacionsPaquet().remove(getComprovacioActiva());
		setIndexLlistaComprovacions(getIndexLlistaComprovacions()-1);
		if (getIndexLlistaComprovacions()<0){ 
			setIndexLlistaComprovacions(0);
			}
		canviarComprovacioActiva();
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Comprovacio
	 * @pre: atribut clonarComprovacio
	 * @post: si clonarComprovacio = true
	 * 				clona comprovacioActiva amb id = 0 (nou registre)
	 * 		  si clonarComprovacio = false
	 * 				comprovacioActiva = comprovaciobuida	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String novaComprovacio(){
		if (!getClonarComprovacio()) {
			setComprovacioActiva(comprovacioServeis.comprovacioBuida());
		} else {
			Comprovacio novac = (Comprovacio) getComprovacioActiva().clone();
			setComprovacioActiva(novac);
			getComprovacioActiva().idNouRegistre();
		}
		getComprovacioActiva().setPaquet(getPaquetActiu());
		setComprovacioActivaBackUp( (Comprovacio) getComprovacioActiva().clone());
		return "paquet";
	}

	//--------------- INSTALACIONS ACCIONS
	
	/**
	 * Actuliatza el registre de instalaci actual per la seva eventual edici
	 * @pre: existeix paquetActiu
	 *       existeix indexLlistaInstalacions
	 * @post: instalacioActiva actualitzada
	 */	
	public void canviarInstalacioActiva(){

		if (getPaquetActiu().getInstalacionsPaquet().size()>0){
			setInstalacioActiva(getPaquetActiu().getInstalacionsPaquet().get(indexLlistaInstalacions));
		} else {
			setInstalacioActiva(getInstalacioBuida());
			getInstalacioActiva().setPaquet(getPaquetActiu());
		}
	}

	/**
	 * Respn a la petici de l'usuari per Editar el registre de Instalaci escollit
	 * @pre: existeix instalacioActiva
	 * @post: Instalaci escollida carregada a la interfcie d'usuari
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarInstalacio(){
		canviarInstalacioActiva();
		return "paquet";			
	}

	/**
	 * Respn a la petici de l'usuari per Persistir la Instalacio activa
	 * @pre: existeix instalacioActiva
	 * @post: s'ha persistit  la Instalacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarInstalacio(){
		try {
			if (paquetActiu.getInstalacionsPaquet() != null) {
				List<Instalacio> llistaInstalacions = paquetActiu.getInstalacionsPaquet();
				if (!(llistaInstalacions.contains(instalacioActiva))) {
					llistaInstalacions.add(instalacioActiva);
				} else {
					instalacioServeis.gravarInstalacio(instalacioActiva);
				}
			} else {
				List<Instalacio> llistaInstalacions = new ArrayList<Instalacio>();	
				paquetActiu.setInstalacionsPaquet(llistaInstalacions);			
				llistaInstalacions.add(instalacioActiva);
			}

		} catch	(Exception e) {
			error("gravarInstalacio",e);
		}	
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la Instalacio activa
	 * @pre: existeix instalacioActiva
	 * @post: s'ha eliminat la Instalacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String esborrarInstalacio(){		
		instalacioServeis.esborrarInstalacio(getInstalacioActiva());
		getPaquetActiu().getInstalacionsPaquet().remove(getInstalacioActiva());
		setIndexLlistaInstalacions(getIndexLlistaInstalacions()-1);
		if (getIndexLlistaInstalacions()<0){ 
			setIndexLlistaInstalacions(0);
			}
		canviarInstalacioActiva();
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Instalacio
	 * @pre: atribut clonarInstalacio
	 * @post: si clonarInstalacio = true
	 * 				clona InstalacioActiva amb id = 0 (nou registre)
	 * 		  si clonarInstalacio = false
	 * 				InstalacioActiva = Instalaciobuida	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String novaInstalacio(){
		if (!getClonarInstalacio()) {
			setInstalacioActiva(instalacioServeis.instalacioBuida());
		} else {
			Instalacio novac = (Instalacio) getInstalacioActiva().clone();
			setInstalacioActiva(novac);
			getInstalacioActiva().idNouRegistre();
		}
		getInstalacioActiva().setPaquet(getPaquetActiu());
		setInstalacioActivaBackUp( (Instalacio) getInstalacioActiva().clone());
		return "paquet";
	}

	//--------------- ACTUALITZACIONS ACCIONS
	
	/**
	 * Actualitza el registre de actualitzaci actual per la seva eventual edici
	 * @pre: existeix paquetActiu
	 *       existeix indexLlistaActualitzacions
	 * @post: cactualitzacioActiva actualitzada
	 */	
	public void canviarActualitzacioActiva(){
		if (getPaquetActiu().getActualitzacionsPaquet().size()>0){
			setActualitzacioActiva(getPaquetActiu().getActualitzacionsPaquet().get(indexLlistaActualitzacions));
		} else {
			setActualitzacioActiva(getActualitzacioBuida());
			getActualitzacioActiva().setPaquet(getPaquetActiu());
		}
	}

	/**
	 * Respn a la petici de l'usuari per Editar el registre de Actualitzaci escollit
	 * @pre: existeix actualitzacioActiva
	 * @post: Actualitzaci escollida carregada a la interfcie d'usuari
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarActualitzacio(){
		canviarActualitzacioActiva();
		return "paquet";			
	}

	/**
	 * Respn a la petici de l'usuari per Persistir la Actualitzacio activa
	 * @pre: existeix actualitzacioActiva
	 * @post: s'ha persistit  la Actualitzacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarActualitzacio(){
		try {
			if (paquetActiu.getActualitzacionsPaquet() != null) {
				List<Actualitzacio> llistaActualitzacions = paquetActiu.getActualitzacionsPaquet();
				if (!(llistaActualitzacions.contains(actualitzacioActiva))) {
					llistaActualitzacions.add(actualitzacioActiva);
				} else {
					actualitzacioServeis.gravarActualitzacio(actualitzacioActiva);
				}
			} else {
				List<Actualitzacio> llistaActualitzacions = new ArrayList<Actualitzacio>();	
				paquetActiu.setActualitzacionsPaquet(llistaActualitzacions);			
				llistaActualitzacions.add(actualitzacioActiva);
			}

		} catch	(Exception e) {
			error("gravarActualitzacio",e);
		}	
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la Actualitzacio activa
	 * @pre: existeix actualitzacioActiva
	 * @post: s'ha eliminat la Actualitzacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String esborrarActualitzacio(){		
		actualitzacioServeis.esborrarActualitzacio(getActualitzacioActiva());
		getPaquetActiu().getActualitzacionsPaquet().remove(getActualitzacioActiva());
		setIndexLlistaActualitzacions(getIndexLlistaActualitzacions()-1);
		if (getIndexLlistaActualitzacions()<0){ 
			setIndexLlistaActualitzacions(0);
			}
		canviarActualitzacioActiva();
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Actualitzacio
	 * @pre: atribut clonarActualitzacio
	 * @post: si clonarActualitzacio = true
	 * 				clona ActualitzacioActiva amb id = 0 (nou registre)
	 * 		  si clonarActualitzacio = false
	 * 				ActualitzacioActiva = Actualitzaciobuida	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String novaActualitzacio(){
		if (!getClonarActualitzacio()) {
			setActualitzacioActiva(actualitzacioServeis.actualitzacioBuida());
		} else {
			Actualitzacio novac = (Actualitzacio) getActualitzacioActiva().clone();
			setActualitzacioActiva(novac);
			getActualitzacioActiva().idNouRegistre();
		}
		getActualitzacioActiva().setPaquet(getPaquetActiu());
		setActualitzacioActivaBackUp( (Actualitzacio) getActualitzacioActiva().clone());
		return "paquet";
	}
	//--------------- DESINSTALACIONS ACCIONS
	
	/**
	 * Actualitza el registre de desinstalaci actual per la seva eventual edici
	 * @pre: existeix paquetActiu
	 *       existeix indexLlistaDesinstalacions
	 * @post: desinstalacioActiva actualitzada
	 */	
	public void canviarDesinstalacioActiva(){
		if (getPaquetActiu().getDesinstalacionsPaquet().size()>0){
			setDesinstalacioActiva(getPaquetActiu().getDesinstalacionsPaquet().get(indexLlistaDesinstalacions));
		} else {
			setDesinstalacioActiva(getDesinstalacioBuida());
			getDesinstalacioActiva().setPaquet(getPaquetActiu());
		}
	}

	/**
	 * Respn a la petici de l'usuari per Editar el registre de Desinstalaci escollit
	 * @pre: existeix desinstalacioActiva
	 * @post: Desinstalaci escollida carregada a la interfcie d'usuari
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarDesinstalacio(){
		canviarDesinstalacioActiva();
		return "paquet";			
	}

	/**
	 * Respn a la petici de l'usuari per Persistir la Desinstalacio activa
	 * @pre: existeix desinstalacioActiva
	 * @post: s'ha persistit  la Desinstalacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarDesinstalacio(){
		try {
			if (paquetActiu.getDesinstalacionsPaquet() != null) {
				List<Desinstalacio> llistaDesinstalacions = paquetActiu.getDesinstalacionsPaquet();
				if (!(llistaDesinstalacions.contains(desinstalacioActiva))) {
					llistaDesinstalacions.add(desinstalacioActiva);
				} else {
					desinstalacioServeis.gravarDesinstalacio(desinstalacioActiva);
				}
			} else {
				List<Desinstalacio> llistaDesinstalacions = new ArrayList<Desinstalacio>();	
				paquetActiu.setDesinstalacionsPaquet(llistaDesinstalacions);			
				llistaDesinstalacions.add(desinstalacioActiva);
			}
		} catch	(Exception e) {
			error("gravarDesinstalacio",e);
		}	
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar la Desinstalacio activa
	 * @pre: existeix desinstalacioActiva
	 * @post: s'ha eliminat la Desinstalacio activa
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String esborrarDesinstalacio(){		
		desinstalacioServeis.esborrarDesinstalacio(getDesinstalacioActiva());
		getPaquetActiu().getDesinstalacionsPaquet().remove(getDesinstalacioActiva());
		setIndexLlistaDesinstalacions(getIndexLlistaDesinstalacions()-1);
		if (getIndexLlistaDesinstalacions()<0){ 
			setIndexLlistaDesinstalacions(0);
			}
		canviarDesinstalacioActiva();
		return "paquet";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Desinstalacio
	 * @pre: atribut clonarDesinstalacio
	 * @post: si clonarDesinstalacio = true
	 * 				clona DesinstalacioActiva amb id = 0 (nou registre)
	 * 		  si clonarDesinstalacio = false
	 * 				DesinstalacioActiva = Desinstalaciobuida	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String novaDesinstalacio(){
		if (!getClonarDesinstalacio()) {
			setDesinstalacioActiva(desinstalacioServeis.desinstalacioBuida());
		} else {
			Desinstalacio novac = (Desinstalacio) getDesinstalacioActiva().clone();
			setDesinstalacioActiva(novac);
			getDesinstalacioActiva().idNouRegistre();
		}
		getDesinstalacioActiva().setPaquet(getPaquetActiu());
		setDesinstalacioActivaBackUp( (Desinstalacio) getDesinstalacioActiva().clone());
		return "paquet";
	}
	/**
	 * Displaya l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	private void error(String metode,Exception e){
		setDisplayError(true);
		metodeError=metode;

		if (e.getCause()==null){
			causaError=e.getStackTrace()[0].toString();
		} else {
			causaError=e.getCause().toString();
		}
	}

	/**
	 * Esborra el display de l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	public void noDisplayError(){
		setDisplayError(false);
	}
	
	//------------- POSTCONSTRUCT
	
	/**
	 * S'executa per inicialitzar les estructures de la pgina
	 */
	@PostConstruct
	private void ompleLlistaPaquets(){
		setLlistaPaquets(paquetServeis.llista(getMostrarInhabilitats()));
		setPaquetBuit(paquetServeis.paquetBuit());
		setIndexLlistaPaquets(0);
		if (getLlistaPaquets().size()>0) {
			setPaquetActiu(getLlistaPaquets().get(getIndexLlistaPaquets()));
		} else {
			setPaquetActiu(getPaquetBuit());
		}
		setPaquetBackUp((Paquet) getPaquetActiu().clone());
		setComprovacioBuida(comprovacioServeis.comprovacioBuida());
		paquetActiu.setComprovacionsPaquet(paquetServeis.llistaComprovacions(paquetActiu));
		canviarComprovacioActiva();
		setInstalacioBuida(instalacioServeis.instalacioBuida());
		paquetActiu.setInstalacionsPaquet(paquetServeis.llistaInstalacions(paquetActiu));
		canviarInstalacioActiva();
		setActualitzacioBuida(actualitzacioServeis.actualitzacioBuida());
		paquetActiu.setActualitzacionsPaquet(paquetServeis.llistaActualitzacions(paquetActiu));
		canviarActualitzacioActiva();
		setDesinstalacioBuida(desinstalacioServeis.desinstalacioBuida());
		paquetActiu.setDesinstalacionsPaquet(paquetServeis.llistaDesinstalacions(paquetActiu));
		canviarDesinstalacioActiva();
	}
}
