package gwpkg;

import java.io.Serializable;
import javax.persistence.*;
import java.util.List;
import javax.ejb.Stateful;

/**
* Classe que representa el model de dades de l'entitat PERFIL <P>
* Una instncia d'aquesta entitat es correspon a un PERFIL de programari
* Inclou tots els paquets que agrupa i aquells perfils dels que depen i dels
* n'hereta els paquets
*/
@Entity
@Stateful
@Table(name="perfil")
public class Perfil implements Serializable {
	private static final long serialVersionUID = 1L;

	/** Codi identificador del perfil. s autonumric. */
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)	
	private int idPerfil;
	/** Descripci del perfil. Descripci general del contingut d'un perfil.
	 * Mxim 100 caracters */
	private String descripcioPerfil="";
	/** Nom del perfil. Descriptor mnemotcnic d'entre 0 i 15 caracters.*/
	private String nomPerfil="";
	/** Estat del paquet. Habilitat/Deshabilitat.
	 * Si est habilitat es mostrar a els grids de selecci de paquets.
	 * Si est deshabilitat no es mostrar als grids de selecci.
	 * L'estat deshabilitat no comporta cap altre comportament.
	 * BOOLEAN */ 
	private Boolean inhabilitat=false;

	/** bi-directional many-to-many association to Paquet */
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(
		name="paquetperfil"
		, joinColumns={
			@JoinColumn(name="idPerfil")
			}
		, inverseJoinColumns={
			@JoinColumn(name="idPaquet")
			}
		)
	private List<Paquet> paquets;

	/** bi-directional many-to-many association to Perfil */
	@ManyToMany(fetch=FetchType.EAGER)
	@JoinTable(
		name="dependenciaperfil"
		, joinColumns={
			@JoinColumn(name="idPerfilDepen")
			}
		, inverseJoinColumns={
			@JoinColumn(name="idPerfil")
			}
		)
	private List<Perfil> perfilsDepen;

	/** bi-directional many-to-many association to Perfil */
	@ManyToMany(mappedBy="perfilsDepen", fetch=FetchType.EAGER)
	private List<Perfil> perfils;
	
	/** bi-directional many-to-many association to Host */
	@ManyToMany(mappedBy="perfils", fetch=FetchType.EAGER)
	private List<Host> hosts;

	/** bi-directional many-to-many association to Distribucio */
	@ManyToMany(mappedBy="perfils", fetch=FetchType.EAGER)
	private List<Distribucio> distribucions;

	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public Perfil() {
    }

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public int getIdPerfil() {
		return this.idPerfil;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setIdPerfil(int idPerfil) {
		this.idPerfil = idPerfil;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getDescripcioPerfil() {
		return this.descripcioPerfil;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDescripcioPerfil(String descripcioPerfil) {
		this.descripcioPerfil = descripcioPerfil;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public String getNomPerfil() {
		return this.nomPerfil;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setNomPerfil(String nomPerfil) {
		this.nomPerfil = nomPerfil;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public Boolean getInhabilitat() {
		return this.inhabilitat;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setInhabilitat(Boolean inhabilitat) {
		this.inhabilitat = inhabilitat;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Paquet> getPaquets() {
		return this.paquets;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setPaquets(List<Paquet> paquets) {
		this.paquets = paquets;
	}

	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Perfil> getPerfilsDepen() {
		return this.perfilsDepen;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setPerfilsDepen(List<Perfil> perfilsDepen) {
		this.perfilsDepen = perfilsDepen;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Perfil> getPerfils() {
		return this.perfils;
	}
	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setPerfils(List<Perfil> perfils) {
		this.perfils = perfils;
	}
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Host> getHosts() {
		return this.hosts;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setHosts(List<Host> hosts) {
		this.hosts = hosts;
	}	
	/**Geter exigit per l'especificaci JPA2/EJB3
	 */
	public List<Distribucio> getDistribucions() {
		return this.distribucions;
	}

	/**Seter exigit per l'especificaci JPA2/EJB3
	 */
	public void setDistribucions(List<Distribucio> distribucions) {
		this.distribucions = distribucions;
	}
	/**
	 * Seteja l'identificador del registre a 0.
	 * S'usa per inserir nous registres a la base de dades
	 */
	public void idNouRegistre(){
		idPerfil=0;
	}

}