package gwpkg;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.ejb.Stateful;
import javax.persistence.PersistenceContext;
import javax.persistence.EntityManager;
import javax.persistence.Query;

/** Implementa les operacions de la lgica de negoci pertinents a l'entitat perfil
 */
@Stateful
public class PerfilServeis implements Serializable{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Entity manager JPA usat per les operacions de persistncia de l'entitat host
	 */
	@PersistenceContext(unitName="gwpkg")
	private EntityManager em;
	
	/**Constructor buit exigit per l'especificaci JPA2/EJB3
	 */
	public PerfilServeis(){
	}
	/**
	 * Crea una instncia de perfil Buida.
	 * Es necessita per donar d'alta nous registres a la base de dades
	 * @post instncia de perfil buida
	 * @return instncia de perfil buida
	 */
	public Perfil perfilBuit(){
		Perfil pf = new Perfil();
		return pf;
	}
	
	/**
	 * Clona el registre actual
	 */
	public Perfil clonarPerfil(Perfil perfilOriginal){
		Perfil perfilClonat = perfilBuit();
		perfilClonat.setIdPerfil(perfilOriginal.getIdPerfil());
		perfilClonat.setInhabilitat(perfilOriginal.getInhabilitat());
		perfilClonat.setNomPerfil(perfilOriginal.getNomPerfil());
		perfilClonat.setDescripcioPerfil(perfilOriginal.getDescripcioPerfil());
		List<Perfil> l = new ArrayList<Perfil>();
		perfilClonat.setPerfilsDepen(l);
		if (perfilOriginal.getPerfilsDepen()!= null){
			perfilClonat.getPerfilsDepen().addAll(perfilOriginal.getPerfilsDepen());
		}
		List<Paquet> p = new ArrayList<Paquet>();
		perfilClonat.setPaquets(p);
		if (perfilOriginal.getPaquets()!= null){
			perfilClonat.getPaquets().addAll(perfilOriginal.getPaquets());
		}
		return perfilClonat;
	}	

	/**
	 * Retorna la llista de perfils existents a la base de dades.
	 * Filtra o no els resultats per aquells que estan inhabilitats.
	 * @pre	Existeix mostrarInhabilitats
	 * @post  List<Perfil> Llista perfils filtrada
	 * @return  List<Perfil>
	 */	
	public List<Perfil> llista(Boolean inhabilitats){
		String strQr = "SELECT pf FROM Perfil pf";		
		if (!inhabilitats){
			strQr = strQr + " WHERE pf.inhabilitat = :MOSTRAR_INHABILITATS";
		}
		Query qr = em.createQuery(strQr);			
		if (!inhabilitats){
			qr.setParameter("MOSTRAR_INHABILITATS",inhabilitats);			
		}
		@SuppressWarnings("unchecked")
		List<Perfil> llistaInicial = qr.getResultList();
		
		return llistaInicial;
	}

	/**
	 * Persisteix una instncia de l'entitat perfil
	 * @pre	Existeix perfilActiu
	 * @post  perfilActiu persistit
	 */	
	public void gravarPerfil(Perfil perfilActiu){
		em.merge(perfilActiu);
		}
	/**
	 * Elimina una instncia de l'entitat perfil
	 * @pre	Existeix perfilActiu
	 * @post  perfilActiu eliminat
	 */	
	public void esborrarPerfil(Perfil perfilActiu){
		em.remove(em.merge(perfilActiu));
		}

}
