package gwpkg;

import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.ejb.EJB;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.PostConstruct;

/**
 * Bean de suport (managed bean) per perfilPage.xhtml.
 * Cont els atributs i mtodes que implementen el comportament de
 * la interfcie d'usuari.
 */
@SessionScoped
@Named("perfilView")
public class PerfilView implements Serializable{
	private static final long serialVersionUID = 1L;
	
	//---------- PERFIL	
	/**
	 * Implementa la lgica de negoci referent a l'entitat Perfil.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private PerfilServeis perfilServeis;
	/**
	 * Instncia de l'entitat Perfil amb la que s'est treballant.
	 */
	@EJB
	private Perfil perfilActiu;
	/**
	 * Implementa la lgica de negoci referent a l'entitat Paquet.
	 * Bsicament cont els mtodes de persistncia a base de dades.
	 */
	@EJB
	private PaquetServeis paquetServeis;
	/**
	 * Instncia de l'entitat Perfil amb la que s'inicialitza un nou registre.
	 */
	private Perfil perfilBuit;
	/**
	 * Instncia de l'entitat Perfil que cont l'ltim estat persistit.
	 * Es fa servir per cancellar les ltimes accions fetes i recuperar l'ltim
	 * estat persistit.
	 */
	private Perfil perfilBackUp;
	
	/**
	 * Apuntador a l'element de la llista de Perfils que cont
	 * la base de dades 
	 */
	private Integer indexLlistaPerfils;
	/**
	 * Apuntador a l'element actiu de la llista de Paquets assignats
	 */	
	private Integer indexLlistaPaquetsAssignats;
	/**
	 * Apuntador a l'element de la llista de Paquets disponibles per assignar
	 */
	private Integer indexLlistaPaquetsDisponibles;
	/**
	 * Apuntador a l'element actiu de la llista de Perfils depenents assignats
	 */	
	private Integer indexLlistaPerfilsDepenents;
	/**
	 * Apuntador a l'element de la llista de Perfils disponibles per assignar
	 */
	private Integer indexLlistaPerfilsDisponibles;
	
	/**
	 * Llista contenint els perfils disponibles a la base de dades.
	 */
	private List<Perfil> llistaPerfils;
	/**
	 * Llista contenint els paquets disponibles a la base de dades.
	 */
	private List<Paquet> paquetsDisponibles;
	/**
	 * Llista contenint els perfils disponibles a la base de dades.
	 */
	private List<Perfil> perfilsDisponibles;
	
	/**
	 * true: A la llista de Perfils disponibles s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 * false: A la llista de Perfils disponibles NO s'inclouen les que tenen
	 * 		l'atribut inhabilitada = true
	 */
	private Boolean mostrarInhabilitats=false;
	/**
	 * true: Al inicialitzar un registre nou es clona el registre actiu
	 */
	private Boolean clonarPerfil=false;
	/** Flag per renderitzar o no la <div> d'errors a la pgina
	 * false: no es mmostra la <div> d'errors. No s'ha capturat cap excepci.
	 */
	private Boolean displayError=false;
	/**
	 * Mtode que ha capturat la excepci.
	 */
	private String metodeError="";
	/**
	 * Causa de la excepci.(e.getCause().toString();)
	 */
	private String causaError="";
	
	
	/**Constructor buit exigit per l'especificaci JSF2
	 */
	public PerfilView(){
	}
	
	// -------- PERFIL GETERS I SETTERS
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getDisplayError(){
		return displayError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setDisplayError(Boolean displayError){
		this.displayError=displayError;
	}
	
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getMetodeError(){
		return metodeError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMetodeError(String metodeError){
		this.metodeError=metodeError;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public String getCausaError(){
		return causaError;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setCausaError(String causaError){
		this.causaError=causaError;
	}

	/**Geter exigit per l'especificaci JSF2
	 */
	public Integer getIndexLlistaPerfils(){
		return indexLlistaPerfils;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfils(Integer indexLlistaPerfils){
		this.indexLlistaPerfils=indexLlistaPerfils;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPaquetsAssignats(Integer indexLlistaPaquetsAssignats){
		this.indexLlistaPaquetsAssignats=indexLlistaPaquetsAssignats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsDepenents(Integer indexLlistaPerfilsDepenents){
		this.indexLlistaPerfilsDepenents=indexLlistaPerfilsDepenents;		
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPerfilsDisponibles(Integer indexLlistaPerfilsDisponibles){
		this.indexLlistaPerfilsDisponibles=indexLlistaPerfilsDisponibles;		
	} 
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setIndexLlistaPaquetsDisponibles(Integer indexLlistaPaquetsDisponibles){
		this.indexLlistaPaquetsDisponibles=indexLlistaPaquetsDisponibles;
	} 
	/**Geter exigit per l'especificaci JSF2
	 */
	public Perfil getPerfilActiu(){
		return perfilActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilActiu(Perfil perfilActiu){
		this.perfilActiu=perfilActiu;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilBuit(Perfil perfilBuit){
		this.perfilBuit=perfilBuit;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Perfil getPerfilBuit(){
		return perfilBuit;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilBackUp(Perfil perfilBackUp){
		this.perfilBackUp=perfilBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Perfil getPerfilBackUp(){
		return perfilBackUp;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Perfil> getLlistaPerfils(){
		return llistaPerfils;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setLlistaPerfils(List<Perfil> llistaPerfils){
		this.llistaPerfils=llistaPerfils;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setMostrarInhabilitats(Boolean mostrarInhabilitats){
		this.mostrarInhabilitats = mostrarInhabilitats;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getMostrarInhabilitats(){
		return this.mostrarInhabilitats;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setClonarPerfil(Boolean clonarPerfil){
		this.clonarPerfil = clonarPerfil;
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public Boolean getClonarPerfil(){
		return this.clonarPerfil;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPaquetsDisponibles(){
		paquetsDisponibles = paquetServeis.llista(false);
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Paquet> getPaquetsDisponibles(){
		return paquetsDisponibles;
	}
	/**Seter exigit per l'especificaci JSF2
	 */
	public void setPerfilsDisponibles(){
		perfilsDisponibles = perfilServeis.llista(false);	
	}
	/**Geter exigit per l'especificaci JSF2
	 */
	public List<Perfil> getPerfilsDisponibles(){
		return perfilsDisponibles;
	}
	
	//--------------- Perfil ACCIONS
	
	/**
	 * Respn a la petici de l'usuari per Editar el Perfil escollit
	 * @pre: existeix llistaPerfils
	 * 		 existeix indexLlistaPerfils
	 * @post: perfilActiu=llistaPerfils(indexLlistaPerfils) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String editarPerfil(){
		try {
			canviaPerfilActiu(getIndexLlistaPerfils());		
		} catch	(Exception e) {
			error("editarPerfil",e);
		}	
		return "perfil";
	}
	
	/**
	 * Respn a la petici de l'usuari per Editar el Perfil escollit
	 * @pre: existeix llistaPerfils
	 * 		 existeix indexLlistaPerfils
	 * @post: perfilActiu=llistaPerfils(indexLlistaPerfils) 
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public void canviaPerfilActiu(Integer indexLlistaPerfils){

		if (indexLlistaPerfils >= 0) {
			setIndexLlistaPerfils(indexLlistaPerfils);
			setPerfilActiu(getLlistaPerfils().get(indexLlistaPerfils));
		} else {
			setIndexLlistaPerfils(0);
			setPerfilActiu(getPerfilBuit());
		}
		setPerfilBackUp(perfilServeis.clonarPerfil(perfilActiu));
	}

	/**
	 * Respn a la petici de l'usuari per Persistir el Perfil actiu
	 * @pre: existeix perfilActiu
	 * @post: s'ha persistit  el perfil actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String gravarPerfil(){

		try {
String pepe=null;
pepe.concat("pepi");
			perfilServeis.gravarPerfil(getPerfilActiu());
			if (getPerfilActiu().getIdPerfil()==0) {
				ompleLlistaPerfils();
			}
		} catch (Exception e) {
			error("gravarPerfil",e);
		}
		return "perfil";
	}

	/**
	 * Respn a la petici de l'usuari per Eliminar el perfil actiu
	 * @pre: existeix perfilActiu
	 * @post: s'ha eliminat el perfil actiu
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String esborrarPerfil(){
		try {
			perfilServeis.esborrarPerfil(getPerfilActiu());
			getLlistaPerfils().remove(getPerfilActiu());
			setIndexLlistaPerfils(getIndexLlistaPerfils()-1);
			canviaPerfilActiu(getIndexLlistaPerfils());
		} catch (Exception e) {
			error("esborrarPerfil",e);
		}
		return "perfil";
	}

	/**
	 * Respn a la petici de l'usuari per Crear una nova instancia de Perfil
	 * @pre: atribut clonarPerfil
	 * @post: si clonarPerfil = true
	 * 				clona perfilActiu amb id = 0 (nou registre)
	 * 		  si clonarPerfil = false
	 * 				perfilActiu = perfilBuit	
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String nouPerfil(){
		try {
			if (!clonarPerfil) {
				setPerfilActiu(perfilServeis.perfilBuit());
			} else {
				Perfil noup = perfilServeis.clonarPerfil(perfilActiu);
				setPerfilActiu(noup);
				getPerfilActiu().idNouRegistre();
			}		
			setPerfilBackUp(perfilServeis.clonarPerfil(perfilActiu));
		} catch (Exception e) {
			error("nouPerfil",e);
		}	
		return "perfil";
	}
	
	/**
	 * Respn a la petici de l'usuari per Cancelar les modificacions fetes a Perfil
	 * desde la ltima vegada que ha estat persistida.
	 * @pre: existeix perfilBackup
	 *       existeix perfilActiu
	 * @post: perfilActiu=perfilBackup
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */
	public String cancelarPerfil(){
		try {
			setPerfilActiu( perfilServeis.clonarPerfil(perfilBackUp));
			getLlistaPerfils().set(getIndexLlistaPerfils(), getPerfilActiu());
		} catch	(Exception e) {
			error("cancelarPerfil",e);
		}	
		return "perfil";
	}
	

	/**
	 * Respn a la petici de l'usuari per Filtrar la llista de Perfils disponibles
	 * segons els criteris especificats.
	 * @pre: existeix llistaPerfils
	 *       existeix mostrarInhabilitats
	 * @post: llistaPerfils filtrada
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String filtraLlistaPerfils(){
		try {
			ompleLlistaPerfils();
		} catch	(Exception e) {
			error("filtraLlistaPerfils",e);
		}	
		return "perfil";	
	}

	/**
	 * Respn a la petici de l'usuari per deassignar un paquet del perfil actiu
	 * @pre: existeix perfilActiu
	 *       existeix indexLlistaPaquetsAssignats
	 * @post: paquet  indexLlistaPaquetsAssignats desassignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String desassignarPaquet(){
		try {
			List<Paquet> llistaPaquets = perfilActiu.getPaquets();
			Paquet paquet = llistaPaquets.get(indexLlistaPaquetsAssignats);
			llistaPaquets.remove(paquet);
		} catch	(Exception e) {
			error("desassignarPaquet",e);
		}	
		return "perfil";
	}
	/**
	 * Respn a la petici de l'usuari per assignar un paquet al perfil actiu
	 * @pre: existeix perfilActiu
	 *       existeix indexLlistaPaquetsDisponibles
	 * @post: paquet  indexLlistaPaquetsDisponibles assignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String assignarPaquet(){
		try {
			Paquet paquet = paquetsDisponibles.get(indexLlistaPaquetsDisponibles);
			if (perfilActiu.getPaquets() != null) {
				List<Paquet> llistaPaquets = perfilActiu.getPaquets();
				if (!(llistaPaquets.contains(paquet))) {
					llistaPaquets.add(paquet);
				}
			} else {
				List<Paquet> llistaPaquets = new ArrayList<Paquet>();	
				perfilActiu.setPaquets(llistaPaquets);			
				llistaPaquets.add(paquet);
			}
		} catch	(Exception e) {
			error("assignarPaquet",e);
		}	
		return "perfil";
	}
	/**
	 * Respn a la petici de l'usuari per deassignar una dependencia de perfil del perfil actiu
	 * @pre: existeix perfilActiu
	 *       existeix indexLlistaPerfilsDepenents
	 * @post: paquet  indexLlistaPerfilsDepenents desassignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String desassignarPerfil(){

		try {
			List<Perfil> llistaPerfils = perfilActiu.getPerfilsDepen();
			Perfil perfil = llistaPerfils.get(indexLlistaPerfilsDepenents);
			llistaPerfils.remove(perfil);
		} catch	(Exception e) {
			error("desassignarPerfil",e);
		}	
		return "perfil";
	}
	/**
	 * Respn a la petici de l'usuari per assignar dependencia d'un perfil al perfil actiu
	 * @pre: existeix perfilActiu
	 *       existeix indexLlistaPerfilsDisponibles
	 * @post: paquet  indexLlistaPerfilsDisponibles assignat
	 * @return String pgina a displayar desprs de l'execuci del mtode.
	 */	
	public String assignarPerfil(){
		try {
			Perfil perfil = perfilsDisponibles.get(indexLlistaPerfilsDisponibles);
			if (perfilActiu.getPerfilsDepen() != null) {
				List<Perfil> llistaPerfils = perfilActiu.getPerfilsDepen();
				if (!(llistaPerfils.contains(perfil))) {
					llistaPerfils.add(perfil);
				}
			} else {
				List<Perfil> llistaPerfils = new ArrayList<Perfil>();	
				perfilActiu.setPerfilsDepen(llistaPerfils);			
				llistaPerfils.add(perfil);
			}

		} catch	(Exception e) {
			error("assignarPerfil",e);
		}	
		return "perfil";
	}
	/**
	 * Displaya l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	private void error(String metode,Exception e){
		setDisplayError(true);
		metodeError=metode;

		if (e.getCause()==null){
			causaError=e.getStackTrace()[0].toString();
		} else {
			causaError=e.getCause().toString();
		}
	}

	/**
	 * Esborra el display de l'error a la pgina quan s'ha produt un error capturat per l'aplicatiu
	 */
	public void noDisplayError(){
		setDisplayError(false);
	}

	//------------- POSTCONSTRUCT
	/**
	 * S'executa per inicialitzar les estructures de la pgina
	 */	
	@PostConstruct
	private void ompleLlistaPerfils(){
		setLlistaPerfils(perfilServeis.llista(getMostrarInhabilitats()));
		setPerfilBuit(perfilServeis.perfilBuit());
		setIndexLlistaPerfils(0);
		if (getLlistaPerfils().size()>0) {
			setPerfilActiu(getLlistaPerfils().get(getIndexLlistaPerfils()));
		} else {
			setPerfilActiu(getPerfilBuit());
		}
		setPerfilBackUp(perfilServeis.clonarPerfil(perfilActiu));
		setPaquetsDisponibles();
		setPerfilsDisponibles();
	}
}
